// ** Constants
var Pi         = Math.PI;
var sqrtPi     = Math.sqrt(Pi);
var sqrtTwo    = Math.sqrt(2);
var PiBy2      = Pi/2;
var wZero      = 1.0e-10; //"almost" zero to avoid rounding error problems
var wXcount = 0;  //diagnostic count

// ----------------------------------------------------------------
// Simpson rule for evaluation of integrals
// evaluates integral of wFunc from wFrom to wTo, dividing the
// domain into wNum sub-intervals
// ----------------------------------------------------------------
function zSimpsonPrint(wFunc,wFrom,wTo,wNum)
{
   zSimpsonCalc(wFunc,wFrom,wTo,wNum,true);
}
function zSimpson(wFunc,wFrom,wTo,wNum)
{
   return zSimpsonCalc(wFunc,wFrom,wTo,wNum,false);
}
function zSimpsonCalc(wFunc,wFrom,wTo,wNum,bPrint)
{
   if (bPrint)
   {
      document.write("<p>Integrating <b>" + wFunc +
         "<br/></b> from <b>" + formatDec (wFrom,3) + "</b> to <b>" + formatDec (wTo,3) +
         "<br/></b> over <b>" + wNum + "</b> sub-intervals (" + (2*wNum+1) + " ordinates)</p>");
      document.write("<table border=1>");
      document.write ("<tr> <td><i>n</i></td> \
              <td width='5'></td>\
              <td><i>x</i><sub><i>n</i>-1</sub></td> \
              <td>f ( <i>x</i><sub><i>n</i>-1</sub> )</td> \
              <td width='5'></td>\
              <td>(<i>x<sub>n</sub></i> + <i>x</i><sub><i>n</i>-1</sub>) / 2</td> \
              <td>f ( (<i>x<sub>n</sub></i> + <i>x</i><sub><i>n</i>-1</sub>) / 2 )</td> \
              <td width='5'></td>\
              <td><i>x<sub>n</sub></i></td> \
              <td>f ( <i>x<sub>n</sub></i> )</td> \
              </tr>");
   }
   var x = wFrom ;
   var wy;
   var wInt = 0;
   for (var ix = 0;ix < wNum ;ix++)
   {
      if (bPrint)
      {
         document.write ("<tr>");
         document.write ("<td>" + (ix+1) + "</td>");
         document.write ("<td> </td>");
      }
      try {wy = eval (wFunc);}
      catch (errVal) {alert("cannot evaluate expression\n\n" + wFunc);return; }
      if (bPrint)
      {
         document.write ("<td>" + formatDec (x,3) + "</td>");
         document.write ("<td>" + formatDec (wy,3) + "</td>");
         document.write ("<td> </td>");
      }
      wInt += wy;
      x += (0.5/wNum)*(wTo - wFrom);
      try {wy = eval (wFunc);}
      catch (errVal) {alert("cannot evaluate expression\n\n" + wFunc); }
      if (bPrint)
      {
         document.write ("<td>" + formatDec (x,3) + "</td>");
         document.write ("<td>" + formatDec (wy,3) + "</td>");
         document.write ("<td> </td>");
      }
      wInt += 4*wy;
      x += (0.5/wNum)*(wTo - wFrom);
      try {wy = eval (wFunc);}
      catch (errVal) {alert("cannot evaluate expression\n\n" + wFunc); }
      if (bPrint)
      {
         document.write ("<td>" + formatDec (x,3) + "</td>");
         document.write ("<td>" + formatDec (wy,3) + "</td>");
         document.write ("</tr>");
      }
      wInt += wy;
   }
   wInt *= ((wTo - wFrom)/(wNum * 6));
   if (bPrint)
   {
      document.write ("<tr><td colspan='3'>Integral = " +
         "</td><td style='font-weight:bold;align:center' colspan='7'>" +
         formatDec (wInt,5) +"</td></tr>");
      document.write("</table>");
   }
   return wInt;
}

// ----------------------------------------------------------------
// Format number to show n decimal places
// ----------------------------------------------------------------
function formatDec (iNum,wDP)
{
   if (wDP == undefined) wDP = 2;
   if (wDP < 0) wDP = 0;
   var wNeg = false;
   if (iNum < 0)
   {
      iNum *= -1;
      wNeg = true;
   }
   var wNum;
   var sNum;
   var strZeros = "00000000000000000000";
   if (wDP > 20) wDP = 20;
   wNum = (1.0 * iNum) + (0.5 * Math.pow(10,-1 * wDP));
   sNum = new String(wNum.toString());
   var ix  = sNum.indexOf("e-");
   if (ix  > -1)
   {
     var wExp = Number(sNum.substr (ix+2));
     tString = new String("0.0000000000000000000000000000000");
     sNum = tString.substr(0,wExp+1) +
        sNum.substr(0,1) + sNum.substr(2,ix-2);
   }
   ix = sNum.indexOf(".");
   if (ix < 0)
      sNum = sNum + "." + strZeros;
   else
      sNum = sNum + strZeros;
   ix = sNum.indexOf(".");
   if (wDP > 0)
      sNum = sNum.substr(0,ix + wDP + 1);
   else
      sNum = sNum.substr(0,ix);
   if (wNeg == true && sNum != 0)
     sNum = "-" + sNum;

   return sNum;
}
// ----------------------------------------------------------------
// Error Function Complement
//    erf = (2/sqrt(pi)) * integral ( e ^ ( - u^2 ), 0, x )
//    erfc = 1 - erf
//    asymptotic relation
//      erfc(x)  = e^(-x^2)/ (2x * sqrt(pi))
//           * ( 1 + Sum ( (2n)! / n! (2x)^2n , 1, infty) )
// ----------------------------------------------------------------
function ErfComp(wVal)
{
   var wTerm = 1;
   var wSum = wTerm;
   var w2ValSq = (2 * wVal * wVal);
   for (ix = 1 ; Math.abs(wTerm) > .000001 && ix < 20 ; ix ++)
   {
      wTerm *=  (1 - 2*ix) / w2ValSq;
      wSum += wTerm;
//      document.write("<br/> wTerm = " + wTerm + " &nbsp; " + " wSum = " + wSum);
   }
   wSum /= (Math.exp(wVal * wVal) * 2 * wVal * sqrtPi);
//   document.write("<p> return = " + wSum);
   return 1 - wSum;
}
// ----------------------------------------------------------------
// Taylor expansion of Error Function
// ----------------------------------------------------------------
function ErfTaylor(wVal)
{
   var wTerm = wVal;
   var wSum = wTerm;
   var wValSq = (wVal * wVal);
   for (ix = 1 ; Math.abs(wTerm) > .000001 && ix < 200 ; ix ++)
   {
      wTerm *=  - wValSq * (2 * ix - 1) /(ix *(2 * ix + 1));
      wSum += wTerm;
//      if (ix < 4 || Math.abs(wTerm) < .0001)
//         document.write("<br/>[" + ix +"] wTerm = " + wTerm + " &nbsp; " + " wSum = " + wSum);
   }
   wSum *= 2 / sqrtPi;
//   document.write("<p> return = " + wSum);
   return wSum;
}
// ----------------------------------------------------------------
// The Error Function (erf)
// ----------------------------------------------------------------
function Erf(wVal)
{
   if (wVal > 4)
      return (ErfComp(wVal));
   else
      return ErfTaylor(wVal);
}
// ----------------------------------------------------------------
// returns the gamma function, for the given value
// ----------------------------------------------------------------
function GammaFunc(z)
{
   var start = 4 + (z - Math.floor(z));
   var x = start - 1;
   // evaluate approx gamma for 4.d where d is decimal part
   var gammax = 
      Math.sqrt(2*Math.PI) 
      * Math.pow(x,x+0.5) 
      * Math.exp(-x) 
      * (1 + (1/(8*x))) 
      * (1 - (1/(8*x*Math.PI)));
   // extrapolate to actual value 
   // using fact: Gamma(n + 1) = n * Gamma(n)
   if (z < 4)
      for (var ix = start - 1; ix >= z - .001; ix--)
         gammax /= ix; 
   if (z >= 5)
      for (var ix = start; ix < z - .001; ix++)
         gammax *= ix; 
   return gammax;
}
// ----------------------------------------------------------------
// returns gamma ( n/2 )  where n is  integer
// ----------------------------------------------------------------
function GammaNby2(wN)
{
   var wTerm = (wN/2) - 1
   var wGamma = 1;
   while (wTerm > wZero)
      wGamma *= wTerm--;
   if (wTerm < 0)
      wGamma *= Math.sqrt(Math.PI);
   return wGamma;
}
// ----------------------------------------------------------------
// returns beta ( n , m ) where n , m are integer
// ----------------------------------------------------------------
function BetaNby2 (wN,wM)
{
   var wA = Math.min(wN,wM)/2 - 1;
   var wB = Math.max(wN,wM)/2 - 1;
   var wD = (wN + wM)/2 - 1 ;
   var wBeta = 1;
   while (wA > wZero)
      wBeta *= wA-- * wB-- / (wD-- * wD--);
   while (wB > wZero && wD > wZero)
      wBeta *=  wB-- / wD--;
   while (wB > wZero)
      wBeta *=  wB-- ;
   while (wD > wZero)
      wBeta /=   wD--;
   if (wA < 0)
      wBeta *= sqrtPi;
   if (wB < 0)
      wBeta *= sqrtPi;
   if (wD < 0)
      wBeta /= sqrtPi;
   return wBeta;
}
// ----------------------------------------------------------------
// does the continuous fraction calculation for the
// regularized incomplete Beta Function
// ----------------------------------------------------------------
function BetaIncCF ( Ix, Ia, Ib )
{
   // document.write("<br/>BetaIncCF(" + Ix + " ," + Ia + " ," + Ib + ")");

   var MAXITER = 100;
   var MINDIFF = 1.0e-8;
   var prevB;
   var prevR;
   var prevT;
   var prevU;
   var prevV;
   var prevSum;
   var wDiff = 1;

   var wSum = 1000; // start value to get beyond first loop
   var wB = - (Ia + Ib) * Ix / (Ia + 1) ; // value of b_1
   var wR = 1;
   var wT = 0;
   var wU = 1;
   var wV = 1;
   var wSign = -1;

   for( var wJ = 2 ; wJ < MAXITER && wDiff > MINDIFF; wJ++) // stopping criterion
   {
      wSign *= -1; // -1 for odd wJ, +1 for even wJ
      prevB = wB;
      prevR = wR;
      prevT = wT;
      prevU = wU;
      prevV = wV;
      prevSum = wSum;

      if (wSign < 0) // odd
      {
         var wM = (wJ - 1) / 2;
         wB = - (Ia + wM) * (Ia + Ib + wM) * Ix / ((Ia + 2 * wM) * (Ia + 2 * wM + 1));
      }
      else // even
      {
         var wM = wJ / 2;
         wB = wM * (Ib - wM) * Ix / ((Ia + 2 * wM) * (Ia + 2 * wM - 1));
      }
      wR = prevR * 1 + prevT * prevB;
      wT = prevR;
      wU = prevU * 1 + prevV * prevB;
      wV = prevU;
      // calculate S_n (if needed)
      wSum = (prevR * ( 1 + wB ) + prevT * prevB) /
            (prevU * ( 1 + wB ) + prevV * prevB);
      // document.write("<br/>[" + wJ + "] &nbsp; " + wSum );
      wDiff = Math.abs(wSum - prevSum);
   }
   wH = wSum * Math.pow(Ix,Ia) * Math.pow(Math.abs(Ix - 1),Ib) / (Ia * BetaNby2(2*Ia,2*Ib));
   // document.write("<br/>BetaIncCF returning " + wH );
   return wH;
}
// ----------------------------------------------------------------
// returns regularized incomplete beta
// ----------------------------------------------------------------
function BetaInc ( Ix, Ia, Ib )
{
   // document.write("<br/>BetaInc(" + Ix + " ," + Ia + " ," + Ib + ")");
   var wBI =100;
   if (Ix < (Ia + 1) / (Ia + Ib + 2))
      wBI = BetaIncCF ( Ix, Ia, Ib );
   else
      wBI = 1 - BetaIncCF ( 1 - Ix, Ib, Ia );
   // document.write("<br/>BetaInc returning " + wBI );
   return wBI;
}

// ----------------------------------------------------------------
// ------------------- Gamma distribution -------------------------
// ----------------------------------------------------------------
// gamma probability density
// ----------------------------------------------------------------
function GammaDens(val,alpha,beta)
{
   if (alpha == 1)
      return (beta *  Math.exp(-val*beta) /
         (GammaFunc(alpha)));
   else
      return (beta * Math.pow(val*beta,alpha - 1) * Math.exp(-val*beta) /
         (GammaFunc(alpha)));
}
// ----------------------------------------------------------------
// gamma distribution
// ----------------------------------------------------------------
function GammaDist(val,alpha,beta)
{
   return GammaOne( beta*val, alpha );
}
// ----------------------------------------------------------------
// the gamma distribution for beta=1
// ----------------------------------------------------------------
function GammaOne(val,alpha)
{
   var wTerm = 1;
   var wSum = 0;
   var alphaWhole = false;
   var alphaHalf = false;
   if (Math.abs(alpha - Math.floor(alpha)) < wZero) alphaWhole = true;
   else if (Math.abs(2*alpha - Math.floor(2*alpha)) < wZero) alphaHalf = true;

   if (val <= 0)
      return 0;
   else if (alphaWhole) // *** exact calculation for alpha +ve integer
   {
      wTerm = 1;
      wSum = 0;
      for ( var ix = 1 ; ix < alpha; ix++ )
      {
         wTerm *= val/ix;
         wSum += wTerm;
      }
      wSum *=  Math.exp(-val);
      return 1 - Math.exp(-val) - wSum ;
   }
   else if (alphaHalf) // *** approximation for alpha half +ve integer
   {
//      document.write("<p></p>");
//      document.write("<br/>Gamma One with: &nbsp; val = " + val + ", alpha = " + alpha + " ");
      if (alpha > 0.5 )
      {
         wTerm = Math.sqrt(val)/(sqrtPi/2);
         wSum = wTerm;
         var ix = 1;
         for ( var ix = 2 ; ix < alpha; ix++ )
         {
            wTerm *=  val/(ix - 0.5);
            wSum += wTerm;
         }
         wSum *=  Math.exp(-val);
      }
      else
         wSum = 0;
      wInt =  Erf(Math.sqrt(val));
//      document.write("<br/>wInt = " + wInt + " wSum = " + wSum + " ");
      return Math.min(wInt - wSum,1);
   }
   else // *** otherwise integrate probability density
   {
      var wFunc = " Math.pow(x," + (alpha - 1) + ") * Math.exp(-x) " +
        "/ (GammaFunc(" + alpha + "))";
      return zSimpson(wFunc,0,val,16);
   }
}

// ----------------------------------------------------------------
// inverse of gamma distribution
// ----------------------------------------------------------------
function GammaInv (prob, alpha, beta, dpl)
{
   if (dpl == undefined) dpl = 4;
   var wFunc = "GammaDist(val, " + alpha + "," + beta + ")";
   return (GenInv(wFunc, prob, dpl));
}
// ----------------------------------------------------------------
// ----------------- Chi-squared distribution ---------------------
// ----------------------------------------------------------------
// Chi-squared probability density
// ----------------------------------------------------------------
function ChiSqDens (val, df)
{
   if (val > 0)
      return Math.pow(Math.sqrt(val),(df-2))*Math.exp(-val/2) /
         (GammaNby2(df) * Math.pow(2,df/2));
   else
      return 0;
}
// ----------------------------------------------------------------
// Chi-squared distribution
// ----------------------------------------------------------------
function ChiSqDist (val, df, wSub)
{
   if (wSub == undefined) wSub = 16;
   if (val > 0)
      return GammaDist(val, df/2, 0.5, wSub);
   else
      return 0;
}
// ----------------------------------------------------------------
// inverse of Chi-squared distribution
// ----------------------------------------------------------------
function ChiSqInv (prob, df, dpl)
{
   if (dpl == undefined) dpl = 4;
   var wFunc = "ChiSqDist(val, " + df + ")";
   return (GenInv(wFunc, prob, dpl));
}
// ----------------------------------------------------------------
// -------------- Standard Normal distribution --------------------
// ----------------------------------------------------------------
// standard normal probability density
// ----------------------------------------------------------------
function StdNormDens (val)
{
   return  Math.exp(-(val*val)/2) / Math.sqrt(2*Math.PI) ;
}
// ----------------------------------------------------------------
// standard normal distribution
// ----------------------------------------------------------------
function StdNormDist (val)
{
   if (val > 0)
      return 0.5 + Erf(val/sqrtTwo)/2;
   else
      return 0.5 -  Erf(-val/sqrtTwo)/2;
}
// ----------------------------------------------------------------
// two sided standard normal test value
//   p(|y| > |val|) when val ~ N(0,1)
// ----------------------------------------------------------------
function StdNorm2Tail (val, wSub)
{
   return 1 - Erf(Math.abs(val)/sqrtTwo);
}
// ----------------------------------------------------------------
// inverse of standard normal distribution
// ----------------------------------------------------------------
function StdNormInv(prob,dpl)
{
   if (dpl == undefined) dpl = 4;
   var wFunc = "StdNormDist(val)";
   return (GenInv(wFunc, prob, dpl));
}
// ----------------------------------------------------------------
// -------------------- Normal distribution -----------------------
// ----------------------------------------------------------------
// normal probability density for given mean and std. dev.
// ----------------------------------------------------------------
function NormDens (val, mean, stDev)
{
   return  StdNormDens ( (val - mean) / stDev ) ;
}
function NormDist (val, mean, stDev)
{
   return  StdNormDist ( (val - mean) / stDev ) ;
}
function Norm2Tail (val, mean, stDev)
{
   return  StdNorm2Tail ( (val - mean) / stDev ) ;
}
function NormMirror (val, mean, stDev)
{
   return  2 * mean - val ;
}
// ----------------------------------------------------------------
// inverse of normal distribution
// ----------------------------------------------------------------
function NormInv(prob,mean,stdev,dpl)
{
   if (dpl == undefined) dpl = 4;
   var wFunc = "NormDist(val," + mean + "," + stdev + ")";
   return (GenInv(wFunc, prob, dpl));
}
// ----------------------------------------------------------------
// ---------------- Exponential distribution ----------------------
// ----------------------------------------------------------------
function ExpDens(val, beta)
{
   return (val < 0 ? 0 : beta * Math.exp( - beta * val ));
}
function ExpDist(val, beta)
{
   return (val < 0 ? 0 : 1 -  Math.exp( - beta * val ));
}
function ExpInv(prob, beta)
{
   return -(Math.log(1 - prob)/beta);
}
// ----------------------------------------------------------------
// ------------------ Uniform distribution ------------------------
// ----------------------------------------------------------------
function UniDens(val,lbd,ubd)
{
   return (val < lbd ? 0 : val < ubd ? 1/(ubd-lbd) : 0);
}
function UniDist(val,lbd,ubd)
{
   return (val < lbd ? 0 : val < ubd ? (val-lbd)/(ubd-lbd) : 1);
}
function UniInv(prob,lbd,ubd)
{
   return (lbd + prob*(ubd-lbd));
}
// ----------------------------------------------------------------
// ---------------- Student's t distribution ----------------------
// ----------------------------------------------------------------
// student's t probability density.
// ----------------------------------------------------------------
function StudTDens (val, df)
{
   return  GammaNby2(df + 1) /
      ( Math.sqrt(df * Pi) * GammaNby2(df) *
      Math.pow( 1 + (val*val/df) , (df + 1)/2 ) );
}
function StudT2Tail (val,df)
{
   var wK = df/(df + (val*val));
   return BetaInc(wK, df/2, 0.5);
}
function StudTDist (val,df)
{
   prob = StudT2Tail(val,df) / 2;
   if (val > 0)
      prob = 1 - prob;
   return prob;
}
function StudTInv (prob, df, dpl)
{
   if (dpl == undefined) dpl = 4;
   var wFunc = "StudTDist(val, " + df + ")";
   return (GenInv(wFunc, prob, dpl));
}
function StudTMirror (val, df)
{
   return  -1 * val ;
}
// ----------------------------------------------------------------
// ---------------- (Fisher's) F distribution ---------------------
// ----------------------------------------------------------------
// F probability density
// ----------------------------------------------------------------
function FDens(val, wD1, wD2)
{
   var wNumerator =
      (wD1/wD2) * GammaNby2(wD1+wD2) *
      Math.pow(wD1*val/wD2 , (wD1/2) -1) ;

   var wDenominator =
      GammaNby2(wD1) * GammaNby2(wD2) *
      Math.pow(1 + (wD1*val/wD2) ,  (wD1 + wD2)/2 ) ;
//   if ( val == (wD1 - 1)/wD1 )
//      alert (wNumerator + "\n" + wDenominator);
   return wNumerator/wDenominator;

}
// ----------------------------------------------------------------
// F distribution
// ----------------------------------------------------------------
function FDist(val, wD1, wD2)
{
   var wK = wD1 * val / (wD1 * val + wD2);
   var wBetaInc = BetaInc( wK, wD1/2, wD2/2);
   return wBetaInc;
}
// ----------------------------------------------------------------
// inverse of F distribution
// ----------------------------------------------------------------
function FInv (prob, wD1, wD2, dpl)
{
   if (dpl == undefined) dpl = 4;

   var wFunc = "FDist(val, " + wD1 + "," + wD2 + ")";
   return (GenInv(wFunc, prob, dpl));
}
// ----------------------------------------------------------------
// Inverse of a function.
// returns x such that wFunc(x) = target, correct to dpl dec. pl.
// ----------------------------------------------------------------
function GenInv(wFunc, target, dpl)
{
   if (dpl == undefined) dpl = 4;
   var accuracy = 0.5 * Math.pow(0.1,dpl);
   var val  = 0;
   var dv = 0.5;
   // first get close
   if (eval(wFunc) < target)
      for (;eval(wFunc) < target;val += dv);
   else
      for (;eval(wFunc) > target;val -= dv);
   // binary search
   for(;dv > accuracy; dv /= 2)
      val = (eval(wFunc) > target) ? val - dv : val + dv;
   // rounding and choose value
   val = Math.floor(val*Math.pow(10,dpl)+0.5)/Math.pow(10,dpl);
   var wErr = Math.abs(eval(wFunc)-target);
   var wVal = val;
   val -= Math.pow(0.1,dpl);
   if (wErr > Math.abs(eval(wFunc)-target))
   {
      wVal = val;
      wErr = Math.abs(eval(wFunc)-target);
   }
   val += 2*Math.pow(0.1,dpl)
   if (wErr > Math.abs(eval(wFunc)-target))
      wVal = val;
   return wVal;
}
// ----------------------------------------------------------------
// ------------------ Binomial distribution -----------------------
// ----------------------------------------------------------------
function BinomProb(val, num, wp)
{
   return DiscreteProb("binom", val, num, wp) ;
}
function BinomDist(val, num, wp)
{
   return DiscreteDist("binom", val, num, wp) ;
}
function BinomInv(target, num, wp)
{
   return DiscreteInv("binom", target, num, wp);
}
function BinomLogLhd(val, num, wp)
{
   var wLL;
   if (val == 0 )
      wLL = -2 * num * Math.log(1 - wp)
   else if (val == num)
      wLL = -2 * num * Math.log(wp)
   else
      wLL = -2 * (  val        * ( Math.log( num*wp         / val         ) ) +
                   (num - val) * ( Math.log( (num - num*wp) / (num - val) ) ) );
   return wLL;
}
function BinomMirror(val, num, wp)
{
   var LLFunc = "BinomLogLhd (val, " + num + "," + wp + ")";
   return GenMirror(LLFunc, val, 0, num, num*wp);
}
// ----------------------------------------------------------------
// --------------- Negative Binomial distribution -----------------
// ----------------------------------------------------------------
function NegBinProb(val, alpha, wp)
{
   return DiscreteProb("negbin", val, alpha, wp) ;
}
function NegBinDist(val, alpha, wp)
{
   return DiscreteDist("negbin", val, alpha, wp) ;
}
function NegBinInv(target, alpha, wp)
{
   return DiscreteInv("negbin", target, alpha, wp);
}
// ----------------------------------------------------------------
// ------------------ Geometric distribution ----------------------
// ----------------------------------------------------------------
function GeomProb(val, wp)
{
   return DiscreteProb("geom", val, wp) ;
}
function GeomDist(val, wp)
{
   return DiscreteDist("geom", val, wp) ;
}
function GeomInv(target, wp)
{
   return DiscreteInv("geom", target, wp);
}
// ----------------------------------------------------------------
// ------------------- Poisson distribution -----------------------
// ----------------------------------------------------------------
function PoissonProb(val, lambda)
{
   return DiscreteProb("poisson", val, lambda) ;
}
function PoissonDist(val, lambda)
{
   return DiscreteDist("poisson", val, lambda) ;
}
function PoissonInv(target, lambda)
{
   return DiscreteInv("poisson", target, lambda);
}
function PoissonLogLhd(val, lambda)
{
   return 2 * (lambda  -  val * ( Math.log(lambda / val) + 1 ));
}
function PoissonMirror(val, lambda)
{
   var LLFunc = "PoissonLogLhd (val, " + lambda + ")";
   return GenMirror(LLFunc, val, 0, 3*lambda, lambda);
}
// ----------------------------------------------------------------
// --------------- Hypergeometric distribution --------------------
// ----------------------------------------------------------------
function HyperProb(val, numb, prop, samp)
{
   if ( val > samp )
      return 0; // no need to spend time working it out!
   else if ( val > prop )
      return 0; // no need to spend time working it out!
   else if (( (samp + prop) > numb ) && ( val < (samp + prop - numb) ))
      return 0;
   else if ( (samp + prop) > numb )
      return DiscreteProb("pseudohyper", (val - samp - prop + numb), numb, prop, samp) ;
   else
      return DiscreteProb("hyper", val, numb, prop, samp) ;
}
function HyperDist(val, numb, prop, samp)
{
   if (samp + prop > numb && val < samp + prop - numb)
      return 0;
   else if ( (samp + prop) > numb )
      return DiscreteDist("pseudohyper", (val - samp - prop + numb), numb, prop, samp) ;
   else
      return DiscreteDist("hyper", val, numb, prop, samp) ;
}
function HyperInv(target, numb, prop, samp)
{
   if ( (samp + prop) > numb )
      return ( DiscreteInv("pseudohyper", target, numb, prop, samp) + samp + prop - numb );
   else
      return DiscreteInv("hyper", target, numb, prop, samp);
}
// ----------------------------------------------------------------
// ------- Generic routines for discrete distributions ------------
// ----------------------------------------------------------------
// Prob (0) for various discrete distributions
// ----------------------------------------------------------------
function DiscreteZero(type, parm1, parm2, parm3)
{
   if (type == "binom")
      return (Math.pow(1-parm2, parm1));
   else if (type == "negbin")
      return Math.pow(parm2, parm1);
   else if (type == "poisson")
      return Math.exp(-parm1);
   else if (type == "geom")
      return parm1;
   else if (type == "hyper")
   {
      var wResult = 1;
      for ( var ix = 0; ix < parm3; ix++)
         wResult *= ( parm1 - parm2 - ix ) / ( parm1 - ix );
      return wResult;
   }
   else if (type == "pseudohyper")
   {
      var wResult = 1;
      for ( var ix = 0; ix < parm1 - parm3 ; ix++)
      {
         wResult *=  ((parm2 - ix)  /  (parm1 - ix)) ;
      }
      return wResult;
   }
   else
      return 1;
}
// ----------------------------------------------------------------
// Prob(x)/Prob(x-1) for various discrete distributions
// ----------------------------------------------------------------
function DiscreteInc(type, ix, parm1, parm2, parm3)
{
   if (type == "binom")
      return (((parm1 - ix + 1) * parm2 ) / (ix * (1 - parm2)));
   else if (type == "negbin")
      return ((parm1 + ix - 1 ) * (1 - parm2) / ix );
   else if (type == "poisson")
      return (parm1 / ix);
   else if (type == "geom")
      return (1 - parm1);
   else if (type == "hyper")
      return (parm2 - ix + 1) * (parm3 - ix + 1) / (ix * (parm1 - parm2 - parm3 + ix));
   else if (type == "pseudohyper")
      return (DiscreteInc("hyper", ix + parm2 + parm3 - parm1, parm1, parm2, parm3));
   else
      return 1;
}
// ----------------------------------------------------------------
// Discrete probability   P(x = val)
// ----------------------------------------------------------------
function DiscreteProb(type, val, parm1, parm2, parm3)
{
    var prob = DiscreteZero(type, parm1, parm2, parm3);
    for (var ix = 1; ix < val + 1; ix++)
       prob *= DiscreteInc(type, ix, parm1, parm2, parm3);
    return prob;
}
// ----------------------------------------------------------------
// Discrete distribution P( x =< val)
// ----------------------------------------------------------------
function DiscreteDist(type, val, parm1, parm2, parm3)
{
   val *= 1; // make sure it's numeric
   var wProb = DiscreteZero(type, parm1, parm2, parm3);
   var wProbSum = wProb;
   for (var ix = 1; ix < (val + 1); ix++)
   {
      wProb *= DiscreteInc(type, ix, parm1, parm2, parm3);
      wProbSum += wProb;
   }
   return wProbSum;
}
// ----------------------------------------------------------------
// inverse of Discrete distribution:  x s.t. P ( x =< val) = target
// ----------------------------------------------------------------
function DiscreteInv(type, target, parm1, parm2, parm3)
{
   target *= 1; // make sure it's numeric
   if (!(target < 1)) return parm1;
   var ix = 0;
   var wProb = DiscreteZero(type, parm1, parm2, parm3);
   var wProbSum = wProb;
   for (ix = 1; wProbSum < target + .0001; ix++)
   {
      wProb *= DiscreteInc(type, ix, parm1, parm2, parm3);
      wProbSum += wProb;
   }
   if (ix > 0) ix--; //correct last increment of loop
   return (ix - 1);
}
// ----------------------------------------------------------------
// General mirror function for discrete distributions.
//   Returns the mirror value (x') of the given test value (x)
//   using the log-likelihood funtion. i.e. the cut-off on the
//   opposite side of the curve, so that
//   log-lhd (y) >= log-lhd (x) for all y >= x' ( if x'>x )
//                           or for all y <= x' ( if x'<x )
// ----------------------------------------------------------------
function GenMirror(LLFunc, testVal, lbd, ubd, minVal)
{
   var val;
   var MirrorVal = 0;

   // --- work out log likelihood for vals either side of val
   // --- and val itself. Remember loglike min at max likelihood

   var LLprev;
   var LLval;
   var LLnext;
   var LLmirror

   if (testVal == lbd) LLprev = 99999;
   else  { val = testVal - 1; LLprev = eval(LLFunc);  }

   val = testVal    ; LLvalue = eval(LLFunc);

   if (testVal == ubd) LLnext = 99999;
   else { val = testVal + 1; LLnext = eval(LLFunc); }

   if (LLprev > LLvalue && LLvalue < LLnext) // testVal at minimum
      MirrorVal = testVal+1;
   else if (LLprev == LLvalue && LLvalue < LLnext) // testVal & prev at minimum
      MirrorVal = testVal-1;
   else if (LLprev > LLvalue && LLvalue == LLnext) // testVal & next at minimum
      MirrorVal = testVal+1;
   else
   {
      // direction of search for mirror value
      var wInc = 1;
      if (LLprev > LLvalue) // i.e. LL decreasing on original side
         wInc = -1; // i.e. move left on mirror side

      // --- select value on other side from minimum to try
      MirrorVal = Math.floor(2*minVal - testVal);
      MirrorVal = Math.max(MirrorVal,lbd);
      MirrorVal = Math.min(MirrorVal,ubd);
      // alert("MirrorVal (start) = " + MirrorVal  );

      var wSign = 1; // +ve => looking for LL <= LLvalue, and vice-versa
      val =  MirrorVal; LLmirror = eval(LLFunc);

      // alert("wInc = " + wInc + ", wSign = " + wSign);
      if ( LLmirror < LLvalue) //i.e. initial try too small
      {
         wSign = -1; // look for LL > LLvalue
         wInc *= -1; // reverse direction of search
      }
      // alert("wInc = " + wInc + ", wSign = " + wSign);

      // --- do search
      while ((wSign*(LLmirror - LLvalue) >= 0) // i.e while LLmirror > LLvalue (+ve)
                                        // or LLmirror < LLvalue (-ve)
           && ( MirrorVal >= lbd && MirrorVal <= ubd)) // emergency stop
      {
         MirrorVal  += wInc;
         val =  MirrorVal; LLmirror = eval(LLFunc);
         // alert(" LL (" + MirrorVal + ") = " + LLmirror + ", ("+ (wSign*(LLmirror - LLvalue)) + ")");
      }
      if (wSign == 1) MirrorVal -= wInc;
   }

   // document.write("<br />MirrorVal =   " + MirrorVal + "</p>");
   return MirrorVal;
}

