   cDataBlock.prototype.VarianceTable        = cDataBlock_VarianceTable;
   cDataBlock.prototype.Model                = cDataBlock_Model;
   cDataBlock.prototype.ModelReduction       = cDataBlock_ModelReduction;

// -------------------------------------------------------------------------
// Data Block Method to print the variances for each factor
// -------------------------------------------------------------------------
function cDataBlock_VarianceTable(iVariableName)
{
   if (!this.Loaded) this.LoadData();
   if (!this.FactStruct) this.BuildFactStruct ();

   // *** identify the variable to be summed
   var varIx = this.CheckVar(iVariableName);
   if (varIx == -1) return;

   document.write("<br/><b>Variance Table</b> for " + iVariableName);
   document.write("<p><table class=\"MCStats\" border=1 \
      cellspacing=0 cellpadding=2>\
      <th> </th>\
      <th>Sum Sq</th>\
      <th>df</th>\
      <th>MS</th>\
      ");
   var ExpSumSq = 0;
   var ExpDegFree = 0;
   for (var facIx = 1; facIx < this.FactorArray.length; facIx++)
   {
      var wSumSq = this.FactorArray[facIx].GetFacSumSq(varIx);
      ExpSumSq += wSumSq;
      var wDegFree = this.FactorArray[facIx].GetDegFree();
      ExpDegFree += wDegFree ;
      var wMeanSq = wSumSq / wDegFree;
      writeRow(this.FactorArray[facIx].Name, formatSig(wSumSq,4),
         wDegFree, formatSig(wMeanSq,4) );
   }
   var TotSumSq = this.GetSumSq(varIx,varIx);
   var TotDegFree = this.FactorArray[0].LevelArray[0].ObsCount - 1;

   var wSumSq = TotSumSq - ExpSumSq;
   var wDegFree = TotDegFree - ExpDegFree;
   var wMeanSq = wSumSq / wDegFree;
   writeRow("Residual", formatSig(wSumSq,4), wDegFree, formatSig(wMeanSq,4));
   var wSumSq = TotSumSq ;
   var wDegFree = TotDegFree;
   var wMeanSq = wSumSq / wDegFree;
   writeRow("Total", formatSig(wSumSq,4), wDegFree, formatSig(wMeanSq,4));
   document.write("</table></p>");
}
// -------------------------------------------------------------------------
// Data Block Method to create a new model.
// -------------------------------------------------------------------------
function cDataBlock_Model (iText)
{
   if (!this.Loaded) this.LoadData();
   return new cModel(iText,this);
}
// -------------------------------------------------------------------------
// Data Block Method to test for model reduction.
// -------------------------------------------------------------------------
function cDataBlock_ModelReduction(iModel1, iModel2)
{
   //** check validity of models
   if (!(iModel1 instanceof cModel))
   {
      alert ("MODEL REDUCTION\n1st parameter is not a model");
      return;
   }
   if (!(iModel2 instanceof cModel))
   {
      alert ("MODEL REDUCTION\n2nd parameter is not a model");
      return;
   }
   if (iModel1.Type != iModel2.Type)
   {
      alert ("MODEL REDUCTION\nModels of different types\nRegression/Anova");
      return;
   }
   if (iModel1.Response != iModel2.Response)
   {
      alert ("MODEL REDUCTION\nBoth models must have same response variable");
      return;
   }
   if (iModel2.ByCategory != undefined)
      if (iModel2.ByCategory != iModel1.ByCategory)
      {
         alert ("MODEL REDUCTION\nModel 2 introduces new 'by category'");
         return;
      }
   if (!iModel1.Constant && iModel2.Constant)
   {
      alert ("MODEL REDUCTION\nModel 2 introduces constant coefficient");
      return;
   }
   for (var var2Ix = 0; var2Ix < iModel2.VariableIx.length; var2Ix++)
   {
      var wFound = false;
      for (var var1Ix = 0; var1Ix < iModel1.VariableIx.length 
         && !wFound; var1Ix++)
      {
         if (iModel2.VariableIx[var2Ix] == iModel1.VariableIx[var1Ix])
            wFound = true;
      }
      if (!wFound)
      {
         alert ("MODEL REDUCTION\nModel 2 not nested in Model 1:\
            \nNew variable - " + iModel2.Variables[var2Ix].Name);
         return;
      }
   }
   for (var fac2Ix = 0; fac2Ix < iModel2.FactorIx.length; fac2Ix++)
   {
      var wFound = false;
      for (var fac1Ix = 0; fac1Ix < iModel1.FactorIx.length 
         && !wFound; fac1Ix++)
      {
         if (iModel2.FactorIx[fac2Ix] == iModel1.FactorIx[fac1Ix])
            wFound = true;
      }
      if (!wFound)
      {
         alert ("MODEL REDUCTION\nModel 2 not nested in Model 1:\
            \nNew factor - " + iModel2.Factors[fac2Ix].Name);
         return;
      }
   }
   var TotSumSq  =  iModel2.GetTotSumSq() - iModel2.GetSumSq();
   var ExpSumSq  =  TotSumSq - (iModel1.GetTotSumSq() - iModel1.GetSumSq());
   var TotDegFree  =  iModel2.GetTotDegFree() - iModel2.GetDegFree();
   var ExpDegFree  =  TotDegFree - 
      (iModel1.GetTotDegFree() - iModel1.GetDegFree());

   document.write ("<p><b>Model Reduction</b> "+
      "<br/>Original Model: &nbsp; " + iModel1.Title() +
      "<br/>Reduced Model: &nbsp; " + iModel2.Title() +
      "</p>");
 
   DrawAnovaTab (ExpSumSq, ExpDegFree, TotSumSq, TotDegFree, "Reduction");
}
// =========================================================================
// Class: cModel
// =========================================================================
function cModel(iText,iData) //* class definition
{
   this.Data = iData;
   this.Type;
   this.Response;
   this.ResponseIx;
   this.Variables = new Array ();
   this.VariableIx = new Array ();
   this.Constant = false;
   this.Factors = new Array();
   this.FactorIx = new Array();
   this.ByCategory;
   this.ByLevel = new Array(); // points to level models
   this.SumSq;    // Model sum of squares
   this.TotSumSq; // Total sum of squares
   this.DegFree;    // Model degrees of freedom
   this.TotDegFree; // Total degrees of freedom

   var wText = new String(iText);

   var wTempArray = wText.split("=");
   this.Response = wTempArray[0];
   this.ResponseIx = this.Data.CheckVar(this.Response);
   if (this.ResponseIx == -1)
      alert("Invalid response variable: " + this.Response);
   wText = new String(wTempArray[1]);

   wTempArray = wText.split("/");
   var wByCategory = wTempArray[1];
   if (wByCategory != undefined)
   {
      var wByCatIx = this.Data.CheckFactor(wByCategory)
      if (wByCatIx == -1)
         alert("Invalid category variable: " + wByCategory);
      else 
         this.ByCategory = this.Data.FactorArray[wByCatIx];
   }
   else
      this.ByCategory = this.Data.FactorArray[0] // NULL factor 
   wText = new String(wTempArray[0]);

   wTempArray = wText.split("+");
   for (var ix = 0; ix < wTempArray.length; ix++)
   {
      var facIx = this.Data.CheckFactor(wTempArray[ix])
      if (facIx != -1)
      {
         this.Factors[this.Factors.length] = this.Data.FactorArray[facIx];
         this.FactorIx[this.FactorIx.length] = facIx;
         delete wTempArray[ix];
      }
      else
      {
         var varIx = this.Data.CheckVar(wTempArray[ix])
         if (varIx != -1)
         {
            this.Variables[this.Variables.length] = 
               this.Data.VariableArray[varIx];
            this.VariableIx[this.VariableIx.length] = varIx;
            delete wTempArray[ix];
         }
      }
   }
   for (var ix = 0; ix < wTempArray.length; ix++)
   {
      if (wTempArray[ix] != undefined)
     {
         if (wTempArray[ix].toUpperCase() == "_C")         
            this.Constant = true;
         else
            alert ("Unknown element " + wTempArray[ix]);
     }
   }
   if (this.Factors.length > 0 && this.Variables.length > 0)
      alert ("Model includes both categorical and quantitive variables." +
         "\nMathCasterStats does not support this type of model");
   if (this.Factors.length > 0 && this.ByCategory.Name != "NULL factor")
   {
      alert ("'By category' has no meaning in an ANOVA model." +
         "\n'By category' will be ignored.");
      this.ByCategory = this.Data.FactorArray[0] // NULL factor 
   }

   if (this.Factors.length > 0)
      this.Type = "ANOVA"
   if (this.Variables.length > 0)
   {
      this.Type = "REG"
      for (var levIx = 0 ; levIx < this.ByCategory.LevelArray.length; levIx++)
      {
         this.ByLevel[this.ByLevel.length] = 
            new cLevelModel(this, this.ByCategory.LevelArray[levIx]);
      }   
   }
   

// *** Methods
   this.Display        = cModel_Display;
   this.Title          = cModel_Title;
   this.GetSumSq       = cModel_GetSumSq;
   this.GetDegFree     = cModel_GetDegFree;
   this.GetTotSumSq    = cModel_GetTotSumSq;
   this.GetTotDegFree  = cModel_GetTotDegFree;
   this.AnovaTab       = cModel_AnovaTab;
   this.RegressTab     = cModel_RegressTab;
   this.CorrelMatrix   = cModel_CorrelMatrix;
}
// -------------------------------------------------------------------------
// Displays Model
// -------------------------------------------------------------------------
function cModel_Display ()
{
   document.write("<p><b>Model:</b> &nbsp; " + this.Title() + "</p>");

   if (this.Type == "ANOVA")
   {
      this.AnovaTab (this.GetSumSq(), this.GetDegFree());
   }
  
   if (this.Type == "REG")
   {
      this.AnovaTab (this.GetSumSq(), this.GetDegFree());
      this.RegressTab ();
   }
}
// -------------------------------------------------------------------------
function cModel_Title ()
{
   wTitle = this.Response + " = ";

   if (this.Type == "ANOVA")
   {
      for (var facIx = 0; facIx < this.Factors.length; facIx++)
      {
         if (facIx > 0) wTitle += " + ";
         wTitle += this.Factors[facIx].Name;
      }
      wTitle +=  " &nbsp; (Analysis of Variance Model)";
   }
  
   if (this.Type == "REG")
   {
      if (this.Constant)
         wTitle += "CONSTANT + "; 
      for (var varIx = 0; varIx < this.Variables.length; varIx++)
      {
         if (varIx > 0) wTitle += " + ";
         wTitle += "b<sub>" + (varIx+1) +  " </sub> * " + this.Variables[varIx].Name;
      }
      wTitle += " &nbsp; (Regression Model)";
      if (this.ByCategory.Name != "NULL factor")
         wTitle += " by " + this.ByCategory.Name;
   }
   return wTitle;
}
// -------------------------------------------------------------------------
// Get the Sum of Squares of the Model ("explained" SS)
// -------------------------------------------------------------------------
function cModel_GetSumSq ()
{
   if (this.SumSq == undefined)
   {
      if (this.Type == "ANOVA")
      {
         this.SumSq = 0;
         for (var ix = 0; ix < this.Factors.length; ix++)
            this.SumSq += this.Factors[ix].GetFacSumSq(this.ResponseIx);
      }
      if (this.Type == "REG")
      {
         this.SumSq = 0;
         for (levIx = 0; levIx < this.ByLevel.length; levIx++)
            this.SumSq += this.ByLevel[levIx].GetSumSq();
      }
   }

   return this.SumSq;

}
// -------------------------------------------------------------------------
// Get the Degrees of Freedom of the Model ("explained" df)
// -------------------------------------------------------------------------
function cModel_GetDegFree ()
{
   if (this.DegFree == undefined)
   {
      if (this.Type == "ANOVA")
      {
      this.DegFree = 0;
      for (var ix = 0; ix < this.Factors.length; ix++)
         this.DegFree += this.Factors[ix].GetDegFree();
      }
      if (this.Type == "REG")
      {
         this.DegFree = -1;
         for (levIx = 0; levIx < this.ByLevel.length; levIx++)
         {
            this.DegFree += this.ByLevel[levIx].GetDegFree();
            if (this.Constant) this.DegFree ++;
         }
      }
   }
   return this.DegFree;
}
// -------------------------------------------------------------------------
// Draw Anova Table for the Model
// -------------------------------------------------------------------------
function cModel_AnovaTab (ExpSumSq, ExpDegFree)
{
   
   var TotSumSq = this.GetTotSumSq();
   var TotDegFree = this.GetTotDegFree();
   DrawAnovaTab (ExpSumSq, ExpDegFree, TotSumSq, TotDegFree);
   if (this.Type == "REG" && !this.Constant)
      document.write("Note: the Sum of Squares and d.f. \
         of the model are calculated for base model: &mu; = 0");
}
// -------------------------------------------------------------------------
// Draw Regression Table for the Model
// -------------------------------------------------------------------------
function cModel_RegressTab ()
{
   for (levIx = 0; levIx < this.ByLevel.length; levIx++)
      this.ByLevel[levIx].RegressTab();
}
// -------------------------------------------------------------------------
// Draw Regression Table for the Model
// -------------------------------------------------------------------------
function cModel_CorrelMatrix ()
{
   for (levIx = 0; levIx < this.ByLevel.length; levIx++)
      this.ByLevel[levIx].CorrelMatrix();
}
// -------------------------------------------------------------------------
// Get the total sum of squares of the Model
// -------------------------------------------------------------------------
function cModel_GetTotSumSq ()
{
   if (this.TotSumSq == undefined)
   {
      if ((this.Type == "REG") && (!this.Constant))
         this.TotSumSq = this.Data.GetUncSumSq(this.ResponseIx,this.ResponseIx);
      else
         this.TotSumSq = this.Data.GetSumSq(this.ResponseIx,this.ResponseIx);
   }
   return this.TotSumSq;
}
// -------------------------------------------------------------------------
// Get the total degrees of freedom of the Model
// -------------------------------------------------------------------------
function cModel_GetTotDegFree ()
{
   if (this.TotDegFree == undefined)
   {
      if ((this.Type == "REG") && (!this.Constant))
         this.TotDegFree = this.Data.GetDegFree() + 1;
      else
         this.TotDegFree = this.Data.GetDegFree();
   }
   return this.TotDegFree;
}
// =========================================================================
// Class: cLevelModel
// data concerning the regression model carried out on each level of the
// ByCategory Factor
// =========================================================================
function cLevelModel(iModel, iLevel) //* class definition
{
   this.Model = iModel;
   this.Level = iLevel;
   this.SumSq;
   this.DegFree; 
   //** regression variables
   this.AtAMatrix;
   this.AtAinvMatrix;
   this.PrMatrix;
   this.BetaMatrix;
   
   this.RegressTab             = cLevelModel_RegressTab;
   this.GetBetaMatrix          = cLevelModel_GetBetaMatrix;
   this.GetAtAMatrix           = cLevelModel_GetAtAMatrix;
   this.GetAtAinvMatrix        = cLevelModel_GetAtAinvMatrix;
   this.GetPrMatrix            = cLevelModel_GetPrMatrix;
   this.GetIntercept           = cLevelModel_GetIntercept;
   this.GetSumSq               = cLevelModel_GetSumSq;
   this.GetDegFree             = cLevelModel_GetDegFree;
   this.CorrelMatrix            = cLevelModel_CorrelMatrix;
}
// -------------------------------------------------------------------------
// Draw Regression Table for the Model at this level
// -------------------------------------------------------------------------
function cLevelModel_RegressTab ()
{
   wBetaArray = ((this.GetBetaMatrix()).Col(1)).XElt;
   
//   zEqn( " PrMatrix eq{20} " + this.PrMatrix.McPrint());
//   zEqn( " BetaMatrix eq{20} " + this.BetaMatrix.McPrint());
//   zEqn( " BetaMatrix^T eq{20} " + this.BetaMatrix.Trans().McPrint());

   if (this.Model.ByCategory.Name != "NULL factor")
      document.write("<p>" + this.Model.ByCategory.Name + 
         " = " + this.Level.Name + "</p>");

   var wTotSumSq = this.Level.GetSumSq(this.Model.ResponseIx,this.Model.ResponseIx);
   var wTotDegFree = this.Level.ObsCount - 1;
   var wResSumSq = wTotSumSq - this.GetSumSq();
   var wResDegFree = wTotDegFree - this.GetDegFree();
   var EstVariance = wResSumSq /wResDegFree;

   document.write("<p><table class=\"MCStats\" border=1 \
      cellspacing=0 cellpadding=2>\
      <th>Variable</th>\
      <th>est. Coeff</th>\
      <th>t-value</th>\
      <th>prob.</th>\
      ");
   if (this.Model.Constant)
   {
      writeRow ("Intercept", formatSig(this.GetIntercept(),4),"&nbsp;","&nbsp;");
   }
   for (var varIx = 0; varIx < this.Model.Variables.length; varIx++)
   {
//      alert ("BetaArray element = " + wBetaArray[varIx]);
//      alert ("Est. Var = " + EstVariance);
//      alert ("ATA inverse element = " + this.GetAtAinvMatrix().XElt[varIx][varIx]);
      var tValue = wBetaArray[varIx]/ Math.sqrt(EstVariance * this.GetAtAinvMatrix().XElt[varIx][varIx]);
      var tProb = StudTDist(tValue,wResDegFree);
      var tProbTxt = tProb < 0.001? "&lt; 0.001":formatSig(tProb,4);
      writeRow (
         this.Model.Variables[varIx].Name,
         formatSig(wBetaArray[varIx],4),
         formatSig(tValue,4),
         tProbTxt
         );
   }
   document.write("</table>");
   document.write("Mean Response: " + formatSig(this.Level.GetMean(this.Model.ResponseIx),4) + "</p>");
}
// -------------------------------------------------------------------------
// Gets the matrix of regression coefficients
// -------------------------------------------------------------------------
function cLevelModel_GetBetaMatrix ()
{
   if (this.BetaMatrix == undefined)
   {
      this.BetaMatrix = (this.GetAtAinvMatrix()).Mult(this.GetPrMatrix());
   }
   return this.BetaMatrix;
}
// -------------------------------------------------------------------------
// Gets the inverse of the matrix of SPD's of independent variables
// -------------------------------------------------------------------------
function cLevelModel_GetAtAinvMatrix ()
{
   if (this.AtAinvMatrix == undefined)
   {
      this.AtAinvMatrix = this.GetAtAMatrix().Inverse();
   }
   return this.AtAinvMatrix;
}
// -------------------------------------------------------------------------
// Gets the inverse of the matrix of SPD's of independent variables
// -------------------------------------------------------------------------
function cLevelModel_GetAtAMatrix ()
{
   if (this.AtAMatrix == undefined)
   {
      Sarray = new Array();
      for (var var1Ix = 0; var1Ix < this.Model.Variables.length; var1Ix++)
      {
         Sarray[var1Ix] = new Array ();
         for (var var2Ix = 0; var2Ix < this.Model.Variables.length; var2Ix++)
         {
            var wVar1Ix = this.Model.VariableIx[var1Ix];
            var wVar2Ix = this.Model.VariableIx[var2Ix];
            if (this.Model.Constant)
               Sarray[var1Ix][var2Ix] = this.Level.GetSumSq(wVar1Ix,wVar2Ix);
            else
               Sarray[var1Ix][var2Ix] = this.Level.GetUncSumSq(wVar1Ix,wVar2Ix);
         }
      }
      this.AtAMatrix = new cMatrix (Sarray);
   }
   return this.AtAMatrix;
}
// -------------------------------------------------------------------------
// Gets the matrix of SPD's of response & independent variables
// -------------------------------------------------------------------------
function cLevelModel_GetPrMatrix ()
{
   if (this.PrMatrix == undefined)
   {
      var Parray = new Array();
      for (var varIx = 0; varIx < this.Model.Variables.length; varIx++)
      {
         var wVarIx = this.Model.VariableIx[varIx];
         Parray[varIx] = new Array ();
         if (this.Model.Constant)
            Parray[varIx][0] = 
               this.Level.GetSumSq(wVarIx,this.Model.ResponseIx);
         else
            Parray[varIx][0] = 
               this.Level.GetUncSumSq(wVarIx,this.Model.ResponseIx);
      }
      this.PrMatrix = new cMatrix (Parray);
   }
   return this.PrMatrix;
}
// -------------------------------------------------------------------------
// Gets the intercept coefficient
// -------------------------------------------------------------------------
function cLevelModel_GetIntercept ()
{
   if (this.Intercept == undefined)
   {
      var wMeansArray = new Array();
      for (var varIx = 0; varIx < this.Model.VariableIx.length; varIx++)
      {
         wMeansArray.push(this.Level.GetMean(this.Model.VariableIx[varIx]));
      }
      var wTempArray = new Array();
      wTempArray[0] = wMeansArray;
      var wMeansMatrix = new cMatrix(wTempArray);
      this.Intercept = this.Level.GetMean(this.Model.ResponseIx) -
           wMeansMatrix.Mult(this.GetBetaMatrix()).Elt(1,1);
   }
   return this.Intercept;
}
// -------------------------------------------------------------------------
// Get the Sum of Squares of the Model ("explained" SS) on this level
// -------------------------------------------------------------------------
function cLevelModel_GetSumSq ()
{
   if (this.SumSq == undefined)
   {
      this.SumSq = 
         this.GetBetaMatrix().Trans().Mult(this.GetPrMatrix()).Elt(1,1);
   }

   return this.SumSq;
}
// -------------------------------------------------------------------------
// Get the Degrees of Freedom of the Model ("explained" df) on this level
// -------------------------------------------------------------------------
function cLevelModel_GetDegFree ()
{
   if (this.DegFree == undefined)
   {
      this.DegFree = this.Model.Variables.length;
   }
   return this.DegFree;
}
// -------------------------------------------------------------------------
// Draws the correlation matrix on this level
// -------------------------------------------------------------------------
function cLevelModel_CorrelMatrix ()
{
   var CovarMatrix = this.GetAtAMatrix();

   document.write("<p><b>Correlation Matrix</b></p>");
   document.write("<p><table class=\"MCStats\" border=1 \
      cellspacing=0 cellpadding=2 style='text-align:right'>");
   for (var rix = 0; rix < CovarMatrix.RowCount; rix++)
   {
      if (rix > 0)
         document.write("<tr><td><b>" + this.Model.Variables[rix].Name +"</b></td>");
      else
         document.write("<tr><th>&nbsp;</th>");
      for (var cix = 0; cix < rix; cix++)
      {
         var wCorrel = CovarMatrix.XElt[rix][cix] / Math.sqrt(CovarMatrix.XElt[rix][rix]*CovarMatrix.XElt[cix][cix]);
         if (Math.abs(wCorrel) > 0.7)
            document.write("<td style='color:red'>" + formatSig(wCorrel,4) + "</td>");
         else
            document.write("<td>" + formatSig(wCorrel,4) + "</td>");
      }
      document.write("<th style='text-align:center'>" + this.Model.Variables[rix].Name +"</th>");
      document.write("</tr>");
   }
   document.write("</table>");
   document.write("</p>");
}
// -------------------------------------------------------------------------
// Draws the ANOVA table
// -------------------------------------------------------------------------
function DrawAnovaTab (ExpSumSq, ExpDegFree, TotSumSq, TotDegFree, ExpTitle)
{
   if (ExpTitle == undefined)
      ExpTitle = "Model";
   var ExpMeanSq = ExpSumSq/ExpDegFree;
   var TotMeanSq = TotSumSq/TotDegFree;
   var ResSumSq = TotSumSq - ExpSumSq;
   var ResDegFree = TotDegFree - ExpDegFree;
   var ResMeanSq = ResSumSq/ResDegFree;

   document.write("<p><table class=\"MCStats\" border=1 \
      cellspacing=0 cellpadding=2>\
      <th> </th>\
      <th>Sum of<br/>Squares</th>\
      <th>Deg.<br/>Free.</th>\
      <th>Mean<br/>Square</th>\
      <th>F-value</th>\
      <th>Signif.<br/>Prob.</th>\
      ");
   var wRatio = ExpMeanSq / ResMeanSq;
   var SigProb = 1 - FDist(wRatio, ExpDegFree, ResDegFree);
   var SigProbTxt;
   if (SigProb < 0.001)
      SigProbTxt = "&lt; 0.001 (0.1%)";
   else
      SigProbTxt = formatDec(SigProb,4) +
         " (" + formatDec(SigProb*100,2) + "%)";

   var ResTitle;
   var TotTitle;
   if (ExpTitle == "Model")
   {
      ResTitle = "Residual";
      TotTitle = "Total";
   }
   else
   {
      ResTitle = "Orig. Residual";
      TotTitle = "New Residual";
   }

   writeRow (ExpTitle, formatSig(ExpSumSq,4), ExpDegFree,
      formatSig(ExpMeanSq,4), formatSig(wRatio,4),SigProbTxt);
   writeRow (ResTitle, formatSig(ResSumSq,4), ResDegFree,
      formatSig(ResMeanSq,4),"&nbsp;","&nbsp;");
   writeRow (TotTitle, formatSig(TotSumSq,4), TotDegFree,
      formatSig(TotMeanSq,4),"&nbsp;","&nbsp;");
   document.write("</table>");
   if (ExpTitle == "Model")
      document.write("Root MS Error: " +   formatSig(Math.sqrt(ResMeanSq),4));
   document.write("</p>");
}

