// =========================================================================
// Class: cDataBlock
// =========================================================================
function cDataBlock(iFile) //*************  class definition ***************
{
   this.InputFile = iFile;            // the name of the input file
   this.CountId;                      // the count variable
   this.FactorArray = new Array();    // an array of cFactor objects
   this.VariableArray = new Array();  // an array of cVariable objects
   this.ElementArray = new Array();   // an array of cElement (XML Tag) objects
   this.Loaded = false;
   this.FactStruct = false;
   this.Mean = new Array;
   this.SumSq = new Array;
   this.DegFree;

   // set up NULL factor as first factor
   this.FactorArray[0] = new cFactor("NULL factor", this);

//** Methods
   this.ViewXML              = cDataBlock_ViewXML;
   this.AddVariables         = cDataBlock_AddVariables;
   this.AddVariable          = cDataBlock_AddVariables;
   this.AddFactors           = cDataBlock_AddFactors;
   this.AddFactor            = cDataBlock_AddFactors;
   this.DefineCount          = cDataBlock_DefineCount;
   this.Summary              = cDataBlock_Summary;
   this.Plot                 = cDataBlock_Plot;
   this.BoxPlot              = cDataBlock_BoxPlot;
   this.CDF                  = cDataBlock_CDF;
   this.Histogram            = cDataBlock_Histogram;
   this.NormalPlot           = cDataBlock_NormalPlot;
   this.Tabulate             = cDataBlock_Tabulate;

   this.FindAddElement       = cDataBlock_FindAddElement;
   this.DisplayVariables     = cDataBlock_DisplayVariables;
   this.LoadData             = cDataBlock_LoadData;
   this.Unload               = cDataBlock_Unload;
   this.CheckVar             = cDataBlock_CheckVar;
   this.CheckFactor          = cDataBlock_CheckFactor;
   this.VarianceTable        = cDataBlock_VarianceTable;
   this.GetMean              = cDataBlock_GetMean;
   this.GetSumSq             = cDataBlock_GetSumSq;
   this.GetUncSumSq          = cDataBlock_GetUncSumSq;
   this.GetDegFree           = cDataBlock_GetDegFree;
   this.FactorStruct         = cDataBlock_FactorStruct;
   this.BuildFactStruct      = cDataBlock_BuildFactStruct;
   this.UniPlot              = cDataBlock_UniPlot;
//   this.ExpectedVals         = cDataBlock_ExpectedVals;
   this.ShowFractiles        = cDataBlock_ShowFractiles;
}
// -------------------------------------------------------------------------
function cDataBlock_ViewXML()
{
   document.write("<br /><a target=anovaxmldata href=\"" + this.InputFile + 
      "\">view the XML data</a>");
}
// -------------------------------------------------------------------------
function cDataBlock_AddVariables(iVariables)
{
   if (this.Loaded) this.Unload();
   var VariableNames = iVariables.split(";");
   for (var iVarIx = 0; iVarIx < VariableNames.length; iVarIx++)
      this.VariableArray[this.VariableArray.length] =
         new cVariable(VariableNames[iVarIx], this);
}
// -------------------------------------------------------------------------
function cDataBlock_DisplayVariables()
{
   document.write("<p>");
   for (var eltIx = 0; eltIx < this.ElementArray.length; eltIx++)
      document.write("<br/>Element[" + eltIx + "] = " + this.ElementArray[eltIx].Tag);
   for (var VarIx = 0; VarIx < this.VariableArray.length; VarIx++)
      document.write("<br/>" + this.VariableArray[VarIx].Name + " = " +
         this.VariableArray[VarIx].Formula);
   document.write("</p>");
}
// -------------------------------------------------------------------------
function cDataBlock_FindAddElement(iEltTag)
{
   var bFound = false;
   var eltIx = 0;
   for (eltIx = 0; (eltIx < this.ElementArray.length) && (!bFound); eltIx++)
      if (this.ElementArray[eltIx].Tag == iEltTag) bFound = true;
   if (!bFound)
   {
      eltIx = this.ElementArray.length;
      this.ElementArray[eltIx]= new cElement(iEltTag);
   }
   else
      eltIx--;
   return eltIx;
}
// -------------------------------------------------------------------------
function cDataBlock_AddFactors(iFactors)
{
   if (this.Loaded) this.Unload();
   var FactorNames = iFactors.split(";");
   for (var iFacIx = 0; iFacIx < FactorNames.length; iFacIx++)
      this.FactorArray[this.FactorArray.length] =
         new cFactor(FactorNames[iFacIx], this);
}
// -------------------------------------------------------------------------
function cDataBlock_DefineCount(iCount)
{
   this.CountId = iCount;
}
// -------------------------------------------------------------------------
function cDataBlock_FactorStruct()
{
   if (!this.FactStruct) this.BuildFactStruct ();
   document.write("<br/><b>Factor Structure</b>");
   for (var facIx = 1; facIx < this.FactorArray.length; facIx++)
   {
      var wFact = this.FactorArray[facIx];
      for (var ix = 0; ix < wFact.LessThanList.length; ix++)
         document.write ("<br/> &nbsp; &nbsp; " + wFact.Name + " &gt; " + 
            wFact.LessThanList[ix].Name);
   }
}
// -------------------------------------------------------------------------
function cDataBlock_BuildFactStruct()
{
   for (var facIx = 0; facIx < this.FactorArray.length; facIx++)
      this.FactorArray[facIx].BuildLTList(this.FactorArray);
   this.FactStruct = true;
}
// -------------------------------------------------------------------------
function cDataBlock_LoadData()
{
   var xmlDoc=new ActiveXObject("Microsoft.XMLDOM")
   xmlDoc.async="false";
   xmlDoc.load(this.InputFile);
   // ** if no variables defined, set up dummy variable based on the count tag
   if (this.ElementArray.length == 0)
      this.AddVariables("dummy=1-(0*" + this.CountId + ")");
   // ** xSize gives the number of "records" in the file
   var xSize = xmlDoc.getElementsByTagName(this.ElementArray[0].Tag).length;
   for (var recIx = 0; recIx < xSize ; recIx++)
   {
      wElementValue = new Array();
      for (var eltIx = 0; eltIx < this.ElementArray.length; eltIx++)
      {
         wElementValue[eltIx] =
            Number(xmlDoc.getElementsByTagName
               (this.ElementArray[eltIx].Tag).item(recIx).text) ;
      }

      for (var varIx = 0; varIx < this.VariableArray.length; varIx++)
      {
         // calculate the count for this record
         var wCount = 1;
         if (this.CountId != undefined)
            wCount = Number(xmlDoc.getElementsByTagName
               (this.CountId).item(recIx).text) ;

         // work out the value of the variable
         wVarValue = eval(this.VariableArray[varIx].Formula);

         // record this for factor depending on the level for this record
         for (var facIx = 0; facIx < this.FactorArray.length; facIx++)
         {
            var wFactor = this.FactorArray[facIx];
            var wLevelName = "";
            if (facIx == 0)
               wLevelName = "&nbsp;";
            else
            {
               for (var compIx = 0; compIx < wFactor.Component.length; compIx++)
               {
                  if (compIx > 0) wLevelName += "-";
                  if (wFactor.Component[compIx].indexOf(":") > -1)
                  {
                     var wCompName = wFactor.Component[compIx].split(":")[0];
                     var tempVar = xmlDoc.getElementsByTagName(wCompName).item(recIx).text;
                     wLevelName += eval(wFactor.DistrRule[compIx]);
                  }
                  else
                     wLevelName += xmlDoc.getElementsByTagName
                        (wFactor.Component[compIx]).item(recIx).text;
               }
            }
            var wLevel = wFactor.FindAddLevel(wLevelName);
            // add the value to the calculated level for this factor.
            if (wCount > 0)
               wLevel.AddValue (varIx, wVarValue, wCount);
         }
      }
   }
   this.Loaded = true;
}
// -------------------------------------------------------------------------
function cDataBlock_Unload()
{
   for (var facIx = 0; facIx < this.FactorArray.length; facIx++)
   {
      var wFactor = this.FactorArray[facIx];
      wFactor.LevelArray = new Array ();
      wFactor.Tabulated = false;
   }
   this.Loaded = false;
}
// -------------------------------------------------------------------------
// Data Block Method to print a contingency table for the given factor
// -------------------------------------------------------------------------
function cDataBlock_Tabulate(iFactor,iTest,iTotals)
{

   if (iTotals == undefined) iTotals = true;
   var wFact1Name;
   var wFact2Name;
   var wCrossName;
   var wFact1Comp; // the component tags of the first factor
   var wFact2Comp; //  ditto -- second
   var wCrossComp; //  ditto -- cross
   var slashIx = iFactor.indexOf("/");
   if (slashIx  > -1)
   {
      wFact1Name = iFactor.substr(0,slashIx);
      wFact2Name = iFactor.substr(slashIx + 1);
   }
   else
   {
      wFact1Name = iFactor;
      wFact2Name = "";
   }

   var wFactIx1 = this.CheckFactor(wFact1Name);
   if (wFactIx1 == -1)
   {
      this.AddFactors(wFact1Name);
      wFact1Comp = wFact1Name;
      this.Unload();
   }
   else
   {
      wFact1Comp = this.FactorArray[wFactIx1].GetComponent();
   }

   if (wFact2Name != "")
   {
      var wFactIx2 = this.CheckFactor(wFact2Name);
      if (wFactIx2 == -1)   // factor 2 found
      {
         this.AddFactors(wFact2Name);
         wFact2Comp = wFact2Name;
         this.Unload();
      }
      else // factor 2 found
      {
         wFact2Comp = this.FactorArray[wFactIx2].GetComponent();
      }
      wCrossName = wFact1Name + "*" + wFact2Name;
      wCrossComp = wFact1Comp + "*" + wFact2Comp;
      var factIx = 0;
      for (factIx = 0; factIx < this.FactorArray.length &&
         this.FactorArray[factIx].GetComponent() != wCrossComp; factIx++);
      if (factIx < this.FactorArray.length)
         wCrossName = this.FactorArray[factIx].Name;
      else
      {
         this.AddFactors(wCrossName + "=" + wCrossComp);
         this.Unload();
      }
   }
   else // factor 2 is not defined
   {
      wCrossName = wFact1Name;
   }



   if (!this.Loaded) this.LoadData();

   var wFactor = this.FactorArray[this.CheckFactor(wCrossName)];
   wFactor.TabFact1 = this.FactorArray[this.CheckFactor(wFact1Name)];
   if (wFact2Name != "")
      wFactor.TabFact2 = this.FactorArray[this.CheckFactor(wFact2Name)];

   wFactor.PreTabulate();
   wFactor.Tabulate("O",iTotals);

   if (iTest)
      wFactor.TableIndTest();

}
// -------------------------------------------------------------------------
// Data Block Method to print a summary of the means for all
// factors (for given variable)
// -------------------------------------------------------------------------
function cDataBlock_Summary (iVariable)
{
   if (!this.Loaded) this.LoadData();

   // *** identify the variable to be summed
   var varIx = this.CheckVar(iVariable);
   if (varIx == -1) return;

   document.write("<p><b>Summary of  " + 
      this.VariableArray[varIx].Name + "</b>");
   if (this.FactorArray.length == 1)
      document.write("<br />There are no factors (apart from NULL factor)");
   else if (this.FactorArray.length > 2)
      document.write("<br />There are " + (this.FactorArray.length - 1) +
         " factors (plus  NULL factor)" );
   else
      document.write("<br />There is one factor (plus  NULL factor)");
   document.write("<p><table class=\"MCStats\" border=1 \
      cellspacing=0 cellpadding=2>\
      <th>Factor</th> <th>level</th> <th>obs.</th> <th>total</th>\
      <th>mean</th> <th>min.</th> <th>max.</th> <th>std.dev.</th>\
      <th>Q1</th> <th>med.</th> <th>Q3</th> \
      ");

   for (var facIx = 0; facIx < this.FactorArray.length; facIx++)
   {
      var wFactor = this.FactorArray[facIx];
      var wSumSq = 0;
      var wDf = 0;
      var wStdDev = 0;
      for (var levIx = 0; levIx < wFactor.LevelArray.length; levIx++)
      {
         wLevel = wFactor.LevelArray[levIx];
         var wQuart = wLevel.GetFractiles(varIx,4);
         document.write ("<tr>");
         if (levIx == 0)
            document.write ("<td rowspan=" + (wFactor.LevelArray.length) +
               " valign=\"top\"" + ">" + wFactor.Name + "</td>" );

         document.write ("<td>" + wLevel.Name + "</td>" +
            "<td>" + wLevel.ObsCount + "</td>" +
            "<td>" + formatSig(wLevel.Total[varIx],4) + "</td>" +
            "<td>" + formatSig(wLevel.GetMean(varIx),4) + "</td>" +
            "<td>" + formatSig(wLevel.Min[varIx],4) + "</td>" +
            "<td>" + formatSig(wLevel.Max[varIx],4) + "</td>" +
            "<td>" + formatSig(wLevel.GetStdDev(varIx,varIx),4) + "</td>" +
            "<td>" + formatSig(wQuart[0],4) + "</td>" +
            "<td>" + formatSig(wQuart[1],4) + "</td>" +
            "<td>" + formatSig(wQuart[2],4) + "</td>" +
            "</tr>");
      }
   }
   document.write("</table></p>");
}
// -------------------------------------------------------------------------
// Calculate and show required fractiles for the given variable on all levs
// -------------------------------------------------------------------------
function cDataBlock_ShowFractiles (iVariable, iFract)
{
   if (iFract == undefined) iFract = 4;
   if (!this.Loaded) this.LoadData();

   // *** identify the variable to be summed
   var varIx = this.CheckVar(iVariable);
   if (varIx == -1) return;

   document.write("<p><table class=\"MCStats\" border=1 \
      cellspacing=0 cellpadding=2>\
      ");
   for (var facIx = 0; facIx < this.FactorArray.length; facIx++)
   {
      var wFactor = this.FactorArray[facIx];
      for (var levIx = 0; levIx < wFactor.LevelArray.length; levIx++)
      {
         var wLevel = wFactor.LevelArray[levIx];
         var fractiles = wLevel.GetFractiles(varIx,iFract);
         document.write ("<tr>");
         if (levIx == 0)
            document.write ("<td rowspan=" + (wFactor.LevelArray.length) +
               " valign=\"top\"" + ">" + wFactor.Name + "</td>" );

         document.write ("<td>" + wLevel.Name + "</td>" +
            "<td>" + wLevel.ObsCount + "</td>" +
            "<td>") ;
         for (var ix=0; ix < fractiles.length; ix++)
           document.write (formatSig(fractiles[ix],4) + ", ");
         document.write ("</td>" +            "</tr>");
      }
   }
   document.write("</table></p>");
   for (var facIx = 0; facIx < this.FactorArray.length; facIx++)
   {
      var wFactor = this.FactorArray[facIx];
      for (var levIx = 0; levIx < wFactor.LevelArray.length; levIx++)
      {
         var wLevel = wFactor.LevelArray[levIx];
         document.write("<p>"+wLevel.SrtString(varIx)+"</p>");
      }
   }


}
// -------------------------------------------------------------------------
// Check existence of factor
// -------------------------------------------------------------------------
function cDataBlock_CheckFactor (iFactor)
{
   var facIx = -1;
   for (facIx = 0; facIx < this.FactorArray.length &&
      this.FactorArray[facIx].Name != iFactor; facIx++);
   if (facIx >= this.FactorArray.length)
   {
//      document.write ("<br /><b>Error: Factor " + iFactor + " not found</b>");
//      this.Display();
      facIx = -1;
   }
   return facIx;
}
// -------------------------------------------------------------------------
// Check existence of variable
// -------------------------------------------------------------------------
function cDataBlock_CheckVar (iVariable)
{
   var varIx = -1;
   for (varIx = 0; varIx < this.VariableArray.length &&
      this.VariableArray[varIx].Name != iVariable; varIx++);
   if (varIx >= this.VariableArray.length)
      varIx = -1;
   return varIx;
}
// -------------------------------------------------------------------------
// Data Block Method to print the variances for each factor
// -------------------------------------------------------------------------
function cDataBlock_VarianceTable(iVariableName)
{
   if (!this.Loaded) this.LoadData();
   if (!this.FactStruct) this.BuildFactStruct ();

   // *** identify the variable to be summed
   var varIx = this.CheckVar(iVariableName);
   if (varIx == -1) return;

   document.write("<br/><b>Variance Table</b> for " + iVariableName);
   document.write("<p><table class=\"MCStats\" border=1 \
      cellspacing=0 cellpadding=2>\
      <th> </th>\
      <th>Sum Sq</th>\
      <th>df</th>\
      <th>MS</th>\
      ");
   var ExpSumSq = 0;
   var ExpDegFree = 0;
   for (var facIx = 1; facIx < this.FactorArray.length; facIx++)
   {
      var wSumSq = this.FactorArray[facIx].GetFacSumSq(varIx);
      ExpSumSq += wSumSq;
      var wDegFree = this.FactorArray[facIx].GetDegFree();
      ExpDegFree += wDegFree ;
      var wMeanSq = wSumSq / wDegFree;
      writeRow(this.FactorArray[facIx].Name, formatSig(wSumSq,4),
         wDegFree, formatSig(wMeanSq,4) );
   }
   var TotSumSq = this.GetSumSq(varIx,varIx);
   var TotDegFree = this.FactorArray[0].LevelArray[0].ObsCount - 1;

   var wSumSq = TotSumSq - ExpSumSq;
   var wDegFree = TotDegFree - ExpDegFree;
   var wMeanSq = wSumSq / wDegFree;
   writeRow("Residual", formatSig(wSumSq,4), wDegFree, formatSig(wMeanSq,4));
   var wSumSq = TotSumSq ;
   var wDegFree = TotDegFree;
   var wMeanSq = wSumSq / wDegFree;
   writeRow("Total", formatSig(wSumSq,4), wDegFree, formatSig(wMeanSq,4));
   document.write("</table></p>");
}
// -------------------------------------------------------------------------
// Data Block Method to get the grand mean for a variable.
// -------------------------------------------------------------------------
function cDataBlock_GetMean (iVar)
{
   if (this.Mean[iVar] == undefined)
      this.Mean[iVar] = this.FactorArray[0].LevelArray[0].GetMean(iVar);
   return this.Mean[iVar];
}
// -------------------------------------------------------------------------
// Data Block Method to get the total degrees of freedom.
// -------------------------------------------------------------------------
function cDataBlock_GetDegFree ()
{
   if (this.DegFree == undefined)
      this.DegFree = this.FactorArray[0].LevelArray[0].ObsCount - 1;
   return this.DegFree;
}
// -------------------------------------------------------------------------
// Data Block Method to get the total (corrected) sum of squares 
// for a variable, or corrected sum of products of two variables
// -------------------------------------------------------------------------
function cDataBlock_GetSumSq (xVarIx, yVarIx)
{
   if (this.SumSq[xVarIx] == undefined)
      this.SumSq[xVarIx] = new Array();
   if (this.SumSq[xVarIx][yVarIx] == undefined)
      this.SumSq[xVarIx][yVarIx] = 
         this.FactorArray[0].LevelArray[0].GetSumSq(xVarIx,yVarIx);
   return this.SumSq[xVarIx][yVarIx];
}
// -------------------------------------------------------------------------
// Data Block Method to get the total uncorrected sum of squares 
// for a variable, or corrected sum of products of two variables
// -------------------------------------------------------------------------
function cDataBlock_GetUncSumSq (xVarIx, yVarIx)
{
   return this.FactorArray[0].LevelArray[0].GetUncSumSq(xVarIx,yVarIx);
}
// -------------------------------------------------------------------------
// Data Block Method to draw various scatter plots.
// argument 1: string
//  "y:x"   - plots variable y (vertical) against variable x (horizontal)
//  "y:x/f" - as above, draws a plot for every level of the factor f
//  "y/f"   - plots distribution of variable y by the levels of factor f
// argument 2: boolean
//  true    - fits the appropriate model, i.e. regression line for 2 vars
//            level means for 1 var.
//
// -------------------------------------------------------------------------
function cDataBlock_Plot(iParm, iFit)
{
   if (!this.Loaded) this.LoadData();

   if (iFit == undefined) iFit = false;
   var wTempArray = iParm.split("/");
   var wFactName = wTempArray[1];
   if (wFactName == undefined) wFactName = "NULL factor";
   var wFactIx = this.CheckFactor(wFactName)
   if (wFactIx == -1)
      alert("Invalid \"by\" factor for plot: " + wFactName);
   else
      this.FactorArray[wFactIx].Plot(wTempArray[0],iFit);
}
// -------------------------------------------------------------------------
// The following three Data Block Methods draw various charts for
// univariate data.
// argument 1: string
//  "y/f"   - plots distribution of variable y by the levels of factor f
//
// -------------------------------------------------------------------------
function cDataBlock_BoxPlot(iParm)
{
   this.UniPlot(iParm, "b");
}
function cDataBlock_CDF(iParm)
{
   this.UniPlot(iParm, "c");
}
function cDataBlock_Histogram(iParm, iInt, iFix)
{
   this.UniPlot(iParm, "h", iInt, iFix);
}
function cDataBlock_NormalPlot(iParm)
{
   this.UniPlot(iParm, "n");
}
// -------------------------------------------------------------------------
// Data Block Method does the checking for the univariate charts.
//
// -------------------------------------------------------------------------
function cDataBlock_UniPlot(iParm, iType, iInt, iFix)
{
   if (!this.Loaded) this.LoadData();

   var wTempArray = iParm.split("/");
   var wFactName = wTempArray[1];
   if (wFactName == undefined) wFactName = "NULL factor";
   var wFactIx = this.CheckFactor(wFactName)
   if (wFactIx == -1)
      alert("Invalid \"by\" factor for plot: " + wFactName);
   else
      this.FactorArray[wFactIx].UniPlot(wTempArray[0], iType, iInt, iFix);
}
/* --- redundant? ------------------------------------------------------------
function cDataBlock_ExpectedVals (iFactor,iVals)
{
   // *** identify the factor to be updated
   var facIx = this.CheckFactor(iFactor);
   if (facIx == -1)
   {
      alert("unrecognised factor: " + iFactor);
      return;
   }
   this.FactorArray[facIx].ExpectedVals(iVals);
}
*/
// =========================================================================
// Class: cVariable
// =========================================================================
function cVariable(iParm, iData) //*************  class definition *********
{
   this.Data = iData;
   this.Name;
   this.Formula;

   var wTempArray = iParm.split("=");
   this.Name = wTempArray[0];
   if (wTempArray.length == 1)
      this.Formula = this.Name;
   else
      this.Formula = wTempArray[1];
   var wFormula = new String (this.Formula);
   // extract tag names from variables
   var Separators = new String(",()+-*/<>?: ");
   wFormArray = new Array();
   while(wFormula != "")
   {
      var wIx;
      var bFound = false;
      for(wIx = 0; (wIx < wFormula.length) && (!bFound); wIx++)
      {
          if (Separators.indexOf(wFormula.charAt(wIx)) > -1)
            bFound = true;
      }
      if (bFound)
      {
         wIx--;
         if (wIx > 0)
         wFormArray.push(wFormula.substr(0,wIx));
         wFormArray.push(wFormula.substr(wIx,1));
         wFormula = wFormula.substr(wIx+1);
      }
      else
      {
         wFormArray.push(wFormula);
         wFormula = ""
      }
   }
   for (var wIx = 0; wIx < wFormArray.length; wIx++)
   {
      if ( (Separators.indexOf(wFormArray[wIx]) == -1)  &&
           (wFormArray[wIx].indexOf("Math.") == -1)     &&
           ( isNaN(wFormArray[wIx]) )                      )
          wFormArray[wIx] = "(wElementValue[" +
             this.Data.FindAddElement(wFormArray[wIx]) + "])";
   }
   this.Formula = "";
   for (var ix=0; ix < wFormArray.length; ix++)
   {
      this.Formula += wFormArray[ix];
   }

}
// =========================================================================
// Class: cElement
//   These are the XML tags used in building the DataBlock
// =========================================================================
function cElement(iTag) //*************  class definition ******************
{
   this.Tag = iTag;
}
// =========================================================================
// Class: cFactor
// Each factor comprises a number of levels, referenced through
// LevelArray.
// =========================================================================
function cFactor(iName, iData) //*************  class definition ***********
{
   this.Name = iName;
   this.Derivation = iName;
   this.Data = iData; // ptr. to the data block to which this factor belongs
   this.Component = new Array();
   this.DistrRule = new Array();
   this.LevelArray = new Array();
   this.LessThanList;
   this.ExpSumSq = new Array();
   this.FacSumSq = new Array();
   this.DegFree;
   this.Tabulated = false;
   this.TabFact1;
   this.TabFact2;
   this.ObsMatrix;
   this.ExpMatrix;

   if (iName != "NULL factor")
   {
      if (iName.indexOf("=") != -1)
      {
        this.Name =  iName.split("=")[0];
        this.Derivation = iName.split("=")[1];
      }
      else
      {
        this.Name =  iName;
        this.Derivation =  iName;
      }
      this.Component = this.Derivation.split("*");
      if (this.Component > 1)
         this.Component = this.Component.sort();
      for (var compIx = 0; compIx < this.Component.length; compIx++)
      {
         if (this.Component[compIx].indexOf(":") > -1)
         {
            var wCompName = this.Component[compIx].split(":")[0];
            var wLev = new Array();
            wLev = (this.Component[compIx].split(":")[1]).split(",");
            if (wLev.length < 3 || Modulus(wLev.length,2) != 1)
            {
              alert("Incorrect factor definition \"" + this.Component[compIx] +
                "\"\nfactorisation string must have odd number of elements:" +
                "\n   level , value , level , value , ... , level");
            }
            else
            {
               this.DistrRule[compIx] = "";
               for ( var ix = 0; ix < wLev.length - 1;)
               {
                  this.DistrRule[compIx] += "tempVar<" + wLev[ix+1] + "?\"" + wLev[ix] + "\":";
                  ix += 2;
               }
               this.DistrRule[compIx] += "\"" + wLev[wLev.length - 1] + "\"";
               //alert(wCondition);
            }
         }

      }
   }

// *** Methods
   this.BuildLTList          = cFactor_BuildLTList;
   this.PrintLTList          = cFactor_PrintLTList;
   this.GreaterThan          = cFactor_GreaterThan;
   this.FindAddLevel         = cFactor_FindAddLevel;
   this.GetFacSumSq          = cFactor_GetFacSumSq;
   this.GetExpSumSq          = cFactor_GetExpSumSq;
   this.GetDegFree           = cFactor_GetDegFree;
   this.Plot                 = cFactor_Plot;
   this.UniPlot              = cFactor_UniPlot;
   this.PreTabulate          = cFactor_PreTabulate;
   this.Tabulate             = cFactor_Tabulate;
   this.XTabulate            = cFactor_XTabulate;
   this.TableIndTest         = cFactor_TableIndTest;
   this.GetComponent         = cFactor_GetComponent;
}
// -------------------------------------------------------------------------
// Make a list of all other factors that are less than this one
// -------------------------------------------------------------------------
function cFactor_BuildLTList (iFactors)
{
   this.LessThanList = new Array();
   var LtIx = 0;
   for (var facIx = 0; facIx < iFactors.length; facIx++)
   {
      if (facIx != 0 && this.GreaterThan(iFactors[facIx]))
      // Note NULL factor excluded
         this.LessThanList[LtIx++] = iFactors[facIx];
   }
}
// -------------------------------------------------------------------------
// Print the list of all other factors that are less than this one
// -------------------------------------------------------------------------
function cFactor_PrintLTList (iFactors)
{
   for (var ix = 0; ix < this.LessThanList.length; ix++)
     document.write (this.LessThanList[ix].Name + ", ");
}
// -------------------------------------------------------------------------
// Decide whether this factor is greater than (i.e. finer) than the
// given factor. This is the case if all the constituent parts of
// the given factor are also constituent parts of this factor.
// (e.g.  A*B*C is greater than A*B and A*C but not B*D.)
// -------------------------------------------------------------------------
function cFactor_GreaterThan (iFactor)
{
   var bGreaterThan = false;
   //* exclude this factor itself as this is strictly greater than
   if (this != iFactor)
   {
      bGreaterThan = true;
      for (var compIx = 0; compIx < iFactor.Component.length && 
         bGreaterThan; compIx++)
      {
         var wFound = false;
         for (var compTx = 0; compTx < this.Component.length &&
            !wFound; compTx++)
         {
            if (iFactor.Component[compIx] == this.Component[compTx])
               wFound = true;
         }
         if (!wFound) bGreaterThan = false;
      }
   }
   return bGreaterThan;
}
// -------------------------------------------------------------------------
// Locates and returns the level with given name, else creates it
// -------------------------------------------------------------------------
function cFactor_FindAddLevel (iLevName)
{
   var wLevel;
   var wFound = false;
   var levIx;
   for (levIx = 0; levIx < this.LevelArray.length && !wFound; levIx++)
   {
      if (this.LevelArray[levIx].Name == iLevName)
      {
         wLevel = this.LevelArray[levIx];
         wFound = true;
      }
   }

   // if not in already identified levels, add a new level
   if (!wFound)
   {
      levIx = this.LevelArray.length;
      wLevel = this.LevelArray[levIx] = new cLevel(iLevName,this);
   }

   return wLevel;
}
// -------------------------------------------------------------------------
// Gets the Factor degrees of freedom for this factor
// -------------------------------------------------------------------------
function cFactor_GetDegFree ()
{
   if (this.DegFree  == undefined)
   {
      this.DegFree  = this.LevelArray.length - 1;
      for (var LtIx = 0; LtIx < this.LessThanList.length; LtIx++)
         this.DegFree -= this.LessThanList[LtIx].GetDegFree();
   }
   return this.DegFree;
}
// -------------------------------------------------------------------------
// Gets the Factor sum of squares of given variables for this factor
// -------------------------------------------------------------------------
function cFactor_GetFacSumSq (varIx)
{
   if (this.FacSumSq [varIx] == undefined)
   {
      this.FacSumSq [varIx] = this.GetExpSumSq(varIx,varIx);
      for (var LtIx = 0; LtIx < this.LessThanList.length; LtIx++)
         this.FacSumSq [varIx] -= this.LessThanList[LtIx].GetFacSumSq(varIx);
   }
   return this.FacSumSq[varIx];
}
// -------------------------------------------------------------------------
// Gets the Explained sum of squares of given variables for this factor
// -------------------------------------------------------------------------
function cFactor_GetExpSumSq (xVarIx, yVarIx)
{
   if (this.ExpSumSq [xVarIx] == undefined)
      this.ExpSumSq [xVarIx] = new Array();
   if (this.ExpSumSq [xVarIx][yVarIx] == undefined)
   {
      this.ExpSumSq [xVarIx][yVarIx] = 0;
      for (var levIx = 0; levIx < this.LevelArray.length; levIx++)
      {
         wLevel = this.LevelArray[levIx];
         this.ExpSumSq [xVarIx][yVarIx] +=
            (wLevel.GetMean (xVarIx) - this.Data.GetMean (xVarIx)) *
            (wLevel.GetMean (yVarIx) - this.Data.GetMean (yVarIx)) *
            wLevel.ObsCount;
      }
   }
   return this.ExpSumSq[xVarIx][yVarIx];

}
// -------------------------------------------------------------------------
// plots the given variable(s) at every level of the factor
// -------------------------------------------------------------------------
function cFactor_Plot (iVars, iFit)
{
   //document.write("<br/> Plot " + iVars  + " for every level of " + this.Name);
   //   if (iFit) document.write (" fitting model.");
   var wTempArray = iVars.split(":");
   var wDepName = wTempArray[0];
   var wDepIx = this.Data.CheckVar(wDepName);
   if (wDepIx == -1)
   {
      alert("Invalid variable name: " + wDepName);
      return;
   }
   var wExpIx;
   var regPlot = false;
   if(wTempArray.length > 1)
   {
      regPlot = true;
      var wExpName = wTempArray[1];
      wExpIx = this.Data.CheckVar(wExpName);
      if (wExpIx == -1)
      {
         alert("Invalid variable name: " + wExpName);
         return;
      }
   }

   var xMin;
   var xMax;
   var yMin;
   var yMax;
   var wNullLev = this.Data.FactorArray[0].LevelArray[0];

   colourList = new Array ("blue","red","green","orange","violet","cyan");
   var wColours = "";
   var wPlot = new cGraph();
   // ---------- if there is an explanatory variable draw a scatter chart
   if(wExpIx != undefined)
   {
      xMin = wNullLev.Min[wExpIx];
      xMax = wNullLev.Max[wExpIx];
      yMin = wNullLev.Min[wDepIx];
      yMax = wNullLev.Max[wDepIx];
      zInitScatter(wPlot,xMin, xMax, yMin, yMax);
      wPlot.setYtitle(this.Data.VariableArray[wDepIx].Name);
      wPlot.setXtitle(this.Data.VariableArray[wExpIx].Name);
      for (var levIx = 0; levIx < this.LevelArray.length; levIx++)
      {
         wPlot.drawScatter(this.LevelArray[levIx].ScatterString(wExpIx, wDepIx ));
         wPlot.addLabel(this.LevelArray[levIx].Name, xMax - (0.2 * (xMax-xMin)), yMax - ((levIx) * 0.05 * (yMax-yMin)));
         if (iFit)
         {
            wPlot.drawPolygon(this.LevelArray[levIx].FitModel(wExpIx, wDepIx, levIx ),false);
            wColours += "," + colourList [ levIx  - (6*Math.floor(levIx/6))];
         }
         wColours += "," + colourList [ levIx  - (6*Math.floor(levIx/6))];
      }
   }
   // ---------- if there is no explanatory variable draw a CDF type scatter chart
   else
   {
      yMin = 0;
      yMax = this.LevelArray.length;
      xMin = wNullLev.Min[wDepIx];
      xMax = wNullLev.Max[wDepIx];
      var xRange = xMax - xMin;
      wPlot.setXrange(xMin - (xRange/10),xMax + (xRange/10));
      wPlot.setXtitle(this.Data.VariableArray[wDepIx].Name);
      if (this.Name != "NULL factor")
        wPlot.setYtitle(this.Name);
      wPlot.suppressYgrid();
      for (var levIx = 0; levIx < this.LevelArray.length; levIx++)
      {
         wPlot.drawScatter(this.LevelArray[levIx].CumulateString( wDepIx, levIx ));
         if (this.Name != "NULL factor")
           wPlot.addLabel(
//           this.Name + ": " +
              this.LevelArray[levIx].Name, xMin - (0.05 * (xMax-xMin)), 1.25*levIx + 0.75);
         if (iFit)
         {
            var wMean = this.LevelArray[levIx].GetMean(wDepIx);
            wPlot.drawLine(wMean, 1.25*levIx, wMean, 1.25*levIx + 1);
            wColours += "," + colourList [ levIx  - (6*Math.floor(levIx/6))];
         }
         wColours += "," + colourList [ levIx  - (6*Math.floor(levIx/6))];
      }
   }
   wColours = wColours.substring(1); // gets rid of first comma
   wPlot.setColours(wColours);
   wPlot.print();
}
// -------------------------------------------------------------------------
// plots the given univariate data at every level of the factor
// -------------------------------------------------------------------------
function cFactor_UniPlot (iVar, iType, iInt, iFix)
{
   var wDepName = iVar;
   var wDepIx = this.Data.CheckVar(wDepName);
   if (wDepIx == -1)
   {
      alert("Invalid variable name: " + wDepName);
      return;
   }
   var wNullLev = this.Data.FactorArray[0].LevelArray[0];

   colourList = new Array ("blue","red","green","orange","violet","cyan");
   var wColours = "";
   var wPlot;
   var yMin = 0;
   var yMax = this.LevelArray.length;
   var xMin = wNullLev.Min[wDepIx];
   var xMax = wNullLev.Max[wDepIx];
   var xRange = xMax - xMin;
   var xViewMin = xMin - (xRange/10);
   var xViewMax = xMax + (xRange/10);
   var xDispRange = xViewMax - xViewMin;
//   xgrid   xmesh
   var xMesh = Math.pow(10, Math.floor(Math.log(xDispRange)/Math.log(10)));
   if (xDispRange/xMesh < 2) xMesh /= 10;

   if ("bc".indexOf(iType) != -1)
   //** then draw a plot for each level on same graph
   {
      wPlot = new cGraph();
      wPlot.setXgrid(xMesh);
      wPlot.setXrange(xViewMin, xViewMax);
      wPlot.setXtitle(this.Data.VariableArray[wDepIx].Name);
      if (this.Name != "NULL factor")
        wPlot.setYtitle(this.Name);
      wPlot.suppressYgrid();
      for (var levIx = 0; levIx < this.LevelArray.length; levIx++)
      {
         if (iType == "b")
         {
           wPlot.drawPolygon(this.LevelArray[levIx].hBoxString(wDepIx, levIx ));
           if (this.Name != "NULL factor")
             wPlot.addLabel(
                this.LevelArray[levIx].Name, xMin - (0.05 * (xMax-xMin)), levIx + 0.75);
         }
         else if (iType == "c")
         {
           wPlot.drawPointStep(this.LevelArray[levIx].CumulateStringStep( wDepIx, levIx, xViewMin, xViewMax ));
           if (this.Name != "NULL factor")
             wPlot.addLabel(
                this.LevelArray[levIx].Name, xMin - (0.05 * (xMax-xMin)), 1.25*levIx + 0.75);
         }
         wColours += "," + colourList [ levIx  - (6*Math.floor(levIx/6))];
      }
      wColours = wColours.substring(1); // gets rid of first comma
      wPlot.setColours(wColours);
      wPlot.print();
   }
   else //** draw a graph for each level
   {
      for (var levIx = 0; levIx < this.LevelArray.length; levIx++)
      {
         wColours = "";
         wPlot = new cGraph();
         wPlot.setXgrid(xMesh);
         wPlot.setXrange(xViewMin, xViewMax);
         wPlot.setXtitle(this.Data.VariableArray[wDepIx].Name);
         if (iType == "h")
         {
           wPlot.setYtitle("Relative Frequency");
           var wHisto = new cHistogram(this.LevelArray[levIx], wDepIx, iInt, iFix)
           // wHisto.Tabulate();
           wPlot.drawPointStep(wHisto.FreqString);
           wPlot.addShade(xViewMin, xViewMax);
           if (this.Name != "NULL factor")
             wPlot.addLabel(this.Name + ": " + this.LevelArray[levIx].Name,
               xMin - (0.05 * (xMax-xMin)), wHisto.MaxFreq);
         }
         if (iType == "n")
         {
           wPlot.setYtitle("Expected Normal");
           wPlot.drawScatter(this.LevelArray[levIx].NormalString( wDepIx));
           if (this.Name != "NULL factor")
             wPlot.addLabel(this.Name + ": " + this.LevelArray[levIx].Name,
               this.LevelArray[levIx].Min[wDepIx],this.LevelArray[levIx].Max[wDepIx]);
         }
         wColours += "," + colourList [ levIx  - (6*Math.floor(levIx/6))];
         wColours = wColours.substring(1); // gets rid of first comma
         wPlot.setColours(wColours);
         wPlot.print();
         document.write(" "); //minimal separator
      }
   }

}
// ----------------------------------------------------------------
// works out the x-range, x-mesh and y-stretch for best display
// ----------------------------------------------------------------
function zInitScatter (wGraph, xMin, xMax, yMin, yMax)
{

   var xRange = xMax - xMin;

   var yRange = yMax - yMin;

   // *** set the yStretch to a power of wPow which covers the x:y ratio
   var wPow = 2;
   var roundLog10q = Math.round(Math.log(xRange/yRange)/Math.log(wPow));
   var yStretch = Math.pow(wPow, roundLog10q);
   // *** if resulting y-display range is more than twice x-display, reduce the stretch
   if (yRange * yStretch / xRange > 2)
      yStretch /= wPow;

   var xDispMin;
   var xDispMax;
   if (yRange * yStretch / xRange > 1)
   {
      xDispMin = (xMin + xMax)/2 - (yRange * yStretch)/1.8;
      xDispMax = (xMin + xMax)/2 + (yRange * yStretch)/1.8;
   }
   else
   {
      xDispMin = xMin - (xRange/10);
      xDispMax = xMax + (xRange/10);
   }
   var xDispRange = xDispMax - xDispMin;

   var xMesh = Math.pow(10, Math.floor(Math.log(xDispRange)/Math.log(10)));
   if (xDispRange/xMesh < 2) xMesh /= 10;

   wGraph.setYstretch(yStretch);
   wGraph.setXrange(xDispMin,xDispMax);
   wGraph.setXgrid(xMesh);
}
// -------------------------------------------------------------------------
// Prepares data for tabulation
// -------------------------------------------------------------------------
function cFactor_PreTabulate ()
{
   if (this.TabFact2 != undefined)
   {
      for (rowIx = 0; rowIx < this.TabFact1.LevelArray.length; rowIx++)
      {
         for (colIx = 0; colIx < this.TabFact2.LevelArray.length; colIx++)
         {
            var CellId = this.TabFact1.LevelArray[rowIx].Name + "-" + this.TabFact2.LevelArray[colIx].Name;
            this.FindAddLevel(CellId);
         }
      }
   }
   this.XTabulate();
   this.Tabulated = true;
}
// -------------------------------------------------------------------------
function cFactor_GetComponent ()
{
   var wString = "";
   for (var ix = 0; ix < this.Component.length;ix++)
   {
      if (ix > 0 )
         wString += "*";
      wString += this.Component[ix];
   }
   return wString;
}
// -------------------------------------------------------------------------
function cFactor_XTabulate ()
{
   var ValTab = new Array();
   for (colIx = 0; colIx < this.TabFact1.LevelArray.length; colIx++)
   {
      if (this.TabFact2 != undefined)
      {
         for (rowIx = 0; rowIx < this.TabFact2.LevelArray.length; rowIx++)
         {
            if (ValTab[rowIx] == undefined)
               ValTab[rowIx] = new Array();
            var CellId = this.TabFact1.LevelArray[colIx].Name + "-" + this.TabFact2.LevelArray[rowIx].Name;
            var bFound = false;
            for (var tabIx = 0; tabIx < this.LevelArray.length && !bFound; tabIx++)
            {
               if (this.LevelArray[tabIx].Name == CellId)
               {
                  ValTab[rowIx][colIx] = this.LevelArray[tabIx].ObsCount;
                  bFound = true;
               }
            }
         }
      }
      else
      {
         if (ValTab[0] == undefined)
            ValTab[0] = new Array();
         ValTab[0][colIx] = this.LevelArray[colIx].ObsCount;
      }
   }
   this.ObsMatrix = new cMatrix(ValTab);
}
// -------------------------------------------------------------------------
// test for independence of rows and columns in a table
// -------------------------------------------------------------------------
function cFactor_TableIndTest (iType)
{
//   zEqn (this.ObsMatrix.McPrint());
   wTemp = "1";
   for(var ix = 1 /*sic*/;ix < this.ObsMatrix.RowCount; ix++)
      wTemp += "/1";
   wRowE = new cMatrix(wTemp);
   wTemp = "1";
   for(var ix = 1 /*sic*/;ix < this.ObsMatrix.ColCount; ix++)
      wTemp += "/1";
   wColE = new cMatrix(wTemp);
   var wSum = wRowE.Trans().Mult(this.ObsMatrix).Mult(wColE).Determinant();
//   zEqn("sum eq{20}" + wSum);
   this.ExpMatrix  = this.ObsMatrix.Mult(wColE).Mult(wRowE.Trans().Mult(this.ObsMatrix)).ScalarMult(1/wSum);
//   zEqn (this.ExpMatrix.McPrint());
   document.write("<p><b>Test for independence of rows and columns</b>");
   document.write("<br/>Expected Values:</p>");
   this.Tabulate("E",true);
   var wDiff = (this.ObsMatrix).Minus(this.ExpMatrix);
//   zEqn (wDiff.McPrint());
   var chiSqMat = wDiff.ElementMult(wDiff).ElementDiv(this.ExpMatrix)
//   zEqn (chiSqMat.McPrint());
   var chiSqVal = wRowE.Trans().Mult(chiSqMat).Mult(wColE).Determinant();
   var wDegFree = (this.ObsMatrix.RowCount - 1) * (this.ObsMatrix.ColCount - 1);
   var wSigProb = 1 - ChiSqDist(chiSqVal,wDegFree);
   document.write("<br/>Significance Probability: " + formatDec(wSigProb,4) + " (" + formatDec(wSigProb*100,2) + "%)");
   document.write("<br/>based on a Chi-square value of " + formatDec(chiSqVal,4) +
      " with "  + wDegFree + " Degrees of freedom. ");
}
// -------------------------------------------------------------------------
// draws tables of counts
//   iType = 'O' - tabulate observed values
//         = 'E' - tabulate expected values
// -------------------------------------------------------------------------
function cFactor_Tabulate (iType,iTotals)
{
   var wPrec = (iType=="E"?2:0);
   var wTotRows = (this.TabFact2 != undefined ? this.TabFact2.LevelArray.length + 4 : 4)
   var wTotCols = this.TabFact1.LevelArray.length + 3;

   document.write("<p><table class=\"MCStats\" border=1 \
      cellspacing=0 cellpadding=2 style=\"text-align:center\">");
   document.write ("<tr><td></td><td colspan=\"" + this.TabFact1.LevelArray.length +
      "\" style=\"font-weight:bold\">" +
      this.TabFact1.Name + "</td>");
   if (iTotals)
   {
      document.write ("<td width=6 rowspan=" + wTotRows +
         "></td><td rowspan = 2>Total");
      if (this.TabFact2 != undefined)
         document.write ("/<br/>" + this.TabFact2.Name );
      document.write ("</td>");
   }
   document.write ("</tr><tr>");
   if (this.TabFact2 != undefined)
      document.write ("<td style=\"font-weight:bold\">" + this.TabFact2.Name + "</td>");
   else
      document.write ("<td></td>");
   for (colIx = 0; colIx < this.TabFact1.LevelArray.length; colIx++)
      document.write("<td>" + this.TabFact1.LevelArray[colIx].Name + "</td>");
   document.write ("</tr>");
   var colTot = new Array();
   var grandTot = 0;
   if (this.TabFact2 != undefined)
   {
      for (rowIx = 0; rowIx < this.TabFact2.LevelArray.length; rowIx++)
      {
         var rowTot = 0
         document.write ("<tr>");
         document.write("<td>" + this.TabFact2.LevelArray[rowIx].Name + "</td>");
         for (colIx = 0; colIx < this.TabFact1.LevelArray.length; colIx++)
         {
            var CellId = this.TabFact1.LevelArray[colIx].Name + "-" + this.TabFact2.LevelArray[rowIx].Name;
            var bFound = false;
            if (colTot[colIx] == undefined)
               colTot[colIx] = 0;
            var cellVal =
               (iType == "E" ? this.ExpMatrix.Elt(rowIx+1,colIx+1)
                               : this.ObsMatrix.Elt(rowIx+1,colIx+1));
            document.write("<td>" + formatDec(cellVal,wPrec) + "</td>");
            rowTot += cellVal;
            colTot[colIx] += cellVal;
            grandTot += cellVal;
         }
         if (iTotals) document.write("<td>" + formatDec(rowTot,wPrec) + "</td>");
         document.write ("</tr>");
      }
   }
   else
   {
      for (colIx = 0; colIx < this.LevelArray.length; colIx++)
      {
         var cellVal =
            (iType == "E" ? this.LevelArray[colIx].ExpCount
                          : this.LevelArray[colIx].ObsCount);
         colTot[colIx] = cellVal;
         grandTot += cellVal;
      }
   }

   if (iTotals)
   {
      document.write ("<tr><td height=8 colspan=" + wTotCols + "></td></tr>");
      if (this.TabFact2 != undefined & iTotals)
         document.write ("<tr><td>Total/" + this.TabFact1.Name + "</td>");
      else
         document.write ("<tr><td></td>");
      for (colIx = 0; colIx < this.TabFact1.LevelArray.length; colIx++)
         document.write("<td>" + formatDec(colTot[colIx],wPrec) + "</td>");
      document.write("<td>" + formatDec(grandTot,wPrec) + "</td>");
      document.write ("</tr>");
   }
   document.write ("</table>");
}
// =========================================================================
// Class: cLevel
// =========================================================================
function cLevel(iName,iFactor) //*************  class definition ***********
{
   this.Name  = iName;
   this.Factor = iFactor;
   this.Value = new Array();   // array of values per variable
   this.Count = new Array();   // count of entries in Value array
   this.Total = new Array();   // total per variable
   this.Mean  = new Array();   // mean per variable
   this.Max   = new Array();   // max per variable
   this.Min   = new Array();   // min per variable
   this.sortVals = new Array();// sorted matrix of values[0], individual[1]
                               // and cummulated[2] counts per variable
   this.ObsCount = 0;          // number of observations on this level
   this.ExpCount = 0;
      // expected number of observations on this level - for chi-square test
   this.SumSq = new Array();
      // table (double array) of product of corrected values for two variables
   this.UncSumSq = new Array();
      // table (double array) of product of uncorrected values for two variables
// Methods
   this.AddValue             = cLevel_AddValue;
   this.SortValues          = cLevel_SortValues;
   this.SrtString           = cLevel_SrtString;
   this.GetMean              = cLevel_GetMean;
   this.GetStdDev            = cLevel_GetStdDev;
   this.GetSumSq             = cLevel_GetSumSq;
   this.GetUncSumSq          = cLevel_GetUncSumSq;
   this.GetFractiles         = cLevel_GetFractiles;
   this.ScatterString        = cLevel_ScatterString;
   this.CumulateString       = cLevel_CumulateString;
   this.CumulateStringStep   = cLevel_CumulateStringStep;
   this.NormalString           = cLevel_NormalString;
   this.BoxString            = cLevel_BoxString;
   this.hBoxString           = cLevel_hBoxString;
   this.FitModel             = cLevel_FitModel;
}
// -------------------------------------------------------------------------
//  - Add the value of the given variable.
//  - Update total using the count
//  - Update max and min of variable at this level
// -------------------------------------------------------------------------
function cLevel_AddValue (varIx, iValue, iCount)
{
   if (this.Value [varIx] == undefined)
      this.Value [varIx] = new Array();
   var valIx = this.Value[varIx].length; //end of array + 1
   if (this.Count [valIx] == undefined)
   {
      this.Count [valIx] = iCount;
      this.ObsCount += iCount;
   }

   this.Value [varIx][valIx] = iValue;
   if (this.Total[varIx] == undefined)
      this.Total[varIx] = 0;
   this.Total[varIx] += (iValue * iCount);

   if (this.Min[varIx] == undefined)
      this.Min[varIx] = iValue;
   if (this.Min[varIx] > iValue)
      this.Min[varIx] = iValue;
   if (this.Max[varIx] == undefined)
      this.Max[varIx] = iValue;
   if (this.Max[varIx] < iValue)
      this.Max[varIx] = iValue;
}
// -------------------------------------------------------------------------
// Gets the mean of given variable on this level
// -------------------------------------------------------------------------
function cLevel_GetMean (varIx)
{
   if (this.Mean [varIx] == undefined)
      this.Mean [varIx] = this.Total[varIx] / this.ObsCount;
   return this.Mean[varIx];
}
// -------------------------------------------------------------------------
// Gets the p-fractiles of given variable on this level
// where p = 1 / divs. e.g. if divs = 4, then the result
// is a table with the 1st 2nd and 3rd quartiles.
// -------------------------------------------------------------------------
function cLevel_GetFractiles (varIx,divs)
{
   var fractiles = new(Array);
   var wZero = 1e-6;


   // --- are there enough observations to work out the fractiles
   // -   there are ( divs - 1 ) fractiles (e.g. three quartiles)
   if ( (divs - 1) > this.ObsCount )
      return fractiles;
   // --- the number of observations between fractiles (can be a fraction)
   var chunksize = ( this.ObsCount - (divs - 1) ) / divs;

   // --- build array of sorted values for the given variable
   this.SortValues(varIx);

   // --- work out the positions of the fractiles in the data
   var fractIx = 0;
   var valIx = 0;
   var wPos = chunksize + 1 ;
//   document.write("<br/> --- Observations : " + this.ObsCount);
   while ( ! (wPos  >  this.ObsCount) )
   {
     var wPos1 = Math.floor (wPos);
     //document.write("" + wPos1 + "/");  // ****** testing
     var wFract = wPos - wPos1;
     // --- now continue along sorted value table to find the wPos1'th
     // -   value. Note sortVals...[2] is the cumulated count
     while ( wPos1 > this.sortVals[varIx][valIx][2] )
        valIx++;
     fractiles[fractIx] = ( 1 - wFract ) * this.sortVals[varIx][valIx][0];
/*
     document.write("<br/>position: " + wPos +
        "  \nwPos1 : " + wPos1 +
        "  \nwFract : " + wFract +
        "  \nvalIx : " + valIx +
        "  \ncumulate : " + this.sortVals[varIx][valIx][2] +
        "  \nvalue : " + this.sortVals[varIx][valIx][0] +
        "  \nfractile : " + fractiles[fractIx]
        );
//*/
     if ( wFract > wZero)
       fractiles[fractIx] +=  wFract * this.sortVals[varIx][valIx+1][0];
     fractIx ++;
     wPos += chunksize + 1;
   }


   return fractiles;
}
// -------------------------------------------------------------------------
// produced a sorted list of values of given variable on this level
// -------------------------------------------------------------------------
function cLevel_SortValues (varIx)
{
   if (this.sortVals[varIx] == undefined)
   {
      this.sortVals[varIx] = new Array ();
      var wSortVals =  new Array ();
      for (var ix=0; ix < this.Count.length; ix++)
      {
         wSortVals[ix] =  new Array (4);
         wSortVals[ix][0] = this.Value[varIx][ix] + 1000000;
         wSortVals[ix][1] = this.Value[varIx][ix];
         wSortVals[ix][2] = this.Count[ix];
      }
      wSortVals = wSortVals.sort();
      var wOldSortVal = -999999;
      var valIx = -1;
      for (var ix = 0; ix < wSortVals.length; ix++)
      {
         if (wSortVals[ix][1] != wOldSortVal)
         {
            wOldSortVal = wSortVals[ix][1];
            valIx++;
            this.sortVals[varIx][valIx] = new Array();
            this.sortVals[varIx][valIx][0] = wSortVals[ix][1];
            this.sortVals[varIx][valIx][1] = wSortVals[ix][2];
         }
         else
            this.sortVals[varIx][valIx][1] += wSortVals[ix][2];
      }
      var cumul = 0;
      for (valIx = 0; valIx < this.sortVals[varIx].length; valIx++)
      {
         cumul += this.sortVals[varIx][valIx][1];
         this.sortVals[varIx][valIx][2] = cumul;
      }
   }
}
// -------------------------------------------------------------------------
// Produces a list (sting) of the sorted values and ccumulative count
// -------------------------------------------------------------------------
function cLevel_SrtString (varIx)
{
   this.SortValues(varIx);
   var ListStr = "";
   for (var valIx=0; valIx < this.sortVals[varIx].length; valIx++)
      ListStr += "" + this.sortVals[varIx][valIx][0] +
         "(" + this.sortVals[varIx][valIx][2] + "), " ;
   return ListStr;
}
// -------------------------------------------------------------------------
// Gets the standard deviation of given variable on this level
// -------------------------------------------------------------------------
function cLevel_GetStdDev (varIx)
{
   var wStdDev = 0;
   if (this.ObsCount > 1)
      wStdDev = Math.sqrt(this.GetSumSq(varIx,varIx) / (this.ObsCount - 1));
   return wStdDev;
}
// -------------------------------------------------------------------------
// Gets the (corrected) sum of squares (or sum of products) of the
// given variable(s) on this level.
// -------------------------------------------------------------------------
function cLevel_GetSumSq (xVarIx, yVarIx)
{
   if (this.SumSq [xVarIx] == undefined)
      this.SumSq [xVarIx] = new Array();
   if (this.SumSq [xVarIx][yVarIx] == undefined)
   {
      this.SumSq [xVarIx][yVarIx] = 0;
      for (var valIx = 0; valIx < this.Value[xVarIx].length; valIx++)
      {
         this.SumSq [xVarIx][yVarIx] +=
            (this.Value[xVarIx][valIx] - this.GetMean(xVarIx)) *
            (this.Value[yVarIx][valIx] - this.GetMean(yVarIx)) *
            this.Count[valIx];
      }
   }
   return this.SumSq[xVarIx][yVarIx];
}
// -------------------------------------------------------------------------
// Gets the (uncorrected) sum of squares (or sum of products) of the
// given variable(s) on this level.
// -------------------------------------------------------------------------
function cLevel_GetUncSumSq (xVarIx, yVarIx)
{
   if (this.UncSumSq [xVarIx] == undefined)
      this.UncSumSq [xVarIx] = new Array();
   if (this.UncSumSq [xVarIx][yVarIx] == undefined)
   {
      this.UncSumSq [xVarIx][yVarIx] = 0;
      for (var valIx = 0; valIx < this.Value[xVarIx].length; valIx++)
      {
         this.UncSumSq [xVarIx][yVarIx] +=
            (this.Value[xVarIx][valIx]) *
            (this.Value[yVarIx][valIx]) *
            this.Count[valIx];
      }
   }
   return this.UncSumSq[xVarIx][yVarIx];
}
// -------------------------------------------------------------------------
// Builds a string of values for a scatter plot.
// -------------------------------------------------------------------------
function cLevel_ScatterString (expIx, depIx)
{
   var scatterString = "";
   for (var valIx = 0; valIx < this.Count.length; valIx++)
   {
      if (scatterString != "")  scatterString +=  ",";
      scatterString +=  this.Value[expIx][valIx] + ", " +
          this.Value[depIx][valIx] ;
   }
   return scatterString;
}
// -------------------------------------------------------------------------
// Builds a string of values for a cumulate frequency plot.
// -------------------------------------------------------------------------
function cLevel_CumulateString ( varIx, wLevIx , xViewMin, xViewMax)
{
   this.SortValues(varIx);
   var Yval = 1.25*wLevIx;
   var scatterString = "";
   if (xViewMin != undefined)
        scatterString +=  xViewMin + ", " + Yval;
   for (var valIx = 0; valIx < this.sortVals[varIx].length; valIx++)
   {
      for (frIx = 0 ; frIx < this.sortVals[varIx][valIx][1] ; frIx++)
      {
        Yval += 1/this.ObsCount;
        if (scatterString != "")  scatterString +=  ",";
        scatterString +=  this.sortVals[varIx][valIx][0] + ", " + Yval;
      }
   }
   if (xViewMax != undefined)
        scatterString +=  "," + xViewMax + ", " + Yval;
   return scatterString;
}
// -------------------------------------------------------------------------
// Builds a string of values for a cumulate frequency plot.
// -------------------------------------------------------------------------
function cLevel_CumulateStringStep ( varIx, wLevIx , xViewMin, xViewMax)
{
   this.SortValues(varIx);
   var Yval = 1.25*wLevIx;
   var scatterString = "";
   var wLastVal = xViewMin;
   if (xViewMin != undefined)
        scatterString +=  xViewMin + ", " + Yval;
   for (var valIx = 0; valIx < this.sortVals[varIx].length; valIx++)
   {
      if (this.sortVals[varIx][valIx][0] != wLastVal)
      {
         if (scatterString != "")  scatterString +=  ",";
         scatterString +=  wLastVal + ", " + Yval;
         wLastVal = this.sortVals[varIx][valIx][0];
      }
      Yval += this.sortVals[varIx][valIx][1]/this.ObsCount;
   }
   if (scatterString != "")  scatterString +=  ",";
   scatterString +=  wLastVal + ", " + Yval;
   if (xViewMax != undefined)
        scatterString +=  "," + xViewMax + ", " + Yval;
   return scatterString;
}
// -------------------------------------------------------------------------
// Builds a string of values for a normal plot.
// -------------------------------------------------------------------------
function cLevel_NormalString ( varIx )
{
   this.SortValues(varIx);
   var cumVal = 0;
   var scatterString = "";
   for (var valIx = 0; valIx < this.sortVals[varIx].length; valIx++)
   {
      cumVal += this.sortVals[varIx][valIx][1]/(2*this.ObsCount);
      var Yval = NormInv (cumVal,this.GetMean(varIx),this.GetStdDev(varIx,varIx));
      if (scatterString != "")  scatterString +=  ",";
      scatterString +=  this.sortVals[varIx][valIx][0] + ", " + Yval;
      cumVal += this.sortVals[varIx][valIx][1]/(2*this.ObsCount);
   }
   return scatterString;
}
// -------------------------------------------------------------------------
// Builds a string of values for a box plot.
// -------------------------------------------------------------------------
function cLevel_BoxString (depIx, wLevIx)
{
   var boxString = "";
   var Xval = wLevIx + 0.5;
   var quartiles = this.GetFractiles(depIx,4);

   boxString +=
      ""  + (Xval+0.00)  + "," + this.Max[depIx] +
      "," + (Xval+0.00)  + "," + (quartiles[2]) +
      "," + (Xval-0.25)  + "," + (quartiles[2]) +
      "," + (Xval-0.25)  + "," + (quartiles[1]) +
      "," + (Xval+0.25)  + "," + (quartiles[1]) +
      "," + (Xval+0.25)  + "," + (quartiles[0]) +
      "," + (Xval+0.00)  + "," + (quartiles[0]) +
      "," + (Xval+0.00)  + "," + this.Min[depIx] +
      "," + (Xval+0.00)  + "," + (quartiles[0]) +
      "," + (Xval-0.25)  + "," + (quartiles[0]) +
      "," + (Xval-0.25)  + "," + (quartiles[1]) +
      "," + (Xval+0.25)  + "," + (quartiles[1]) +
      "," + (Xval+0.25)  + "," + (quartiles[2]) +
      "," + (Xval+0.00)  + "," + (quartiles[2]) +
      "," + (Xval+0.00)  + "," + this.Max[depIx] +
      "";


   return boxString;
}
// -------------------------------------------------------------------------
// Builds a string of values for a horizontal box plot.
// -------------------------------------------------------------------------
function cLevel_hBoxString (varIx, wLevIx)
{
   var boxString = "";
   var Yval = wLevIx + 0.5;
   var quartiles = this.GetFractiles(varIx,4);

   boxString +=
      ""  + this.Max[varIx] + "," + (Yval+0.00) +
      "," + (quartiles[2])  + "," + (Yval+0.00) +
      "," + (quartiles[2])  + "," + (Yval-0.25) +
      "," + (quartiles[1])  + "," + (Yval-0.25) +
      "," + (quartiles[1])  + "," + (Yval+0.25) +
      "," + (quartiles[0])  + "," + (Yval+0.25) +
      "," + (quartiles[0])  + "," + (Yval+0.00) +
      "," + this.Min[varIx] + "," + (Yval+0.00) +
      "," + (quartiles[0])  + "," + (Yval+0.00) +
      "," + (quartiles[0])  + "," + (Yval-0.25) +
      "," + (quartiles[1])  + "," + (Yval-0.25) +
      "," + (quartiles[1])  + "," + (Yval+0.25) +
      "," + (quartiles[2])  + "," + (Yval+0.25) +
      "," + (quartiles[2])  + "," + (Yval+0.00) +
      "," + this.Max[varIx] + "," + (Yval+0.00) +
      "";


   return boxString;
}
// -------------------------------------------------------------------------
// Returns the end points of the straight line fitting the data
// -------------------------------------------------------------------------
function cLevel_FitModel (expIx, depIx, wPos)
{
//   document.write ("<br/><b>" + this.Factor.Name + ": " + this.Name + "</b>" );

   var yMean = this.GetMean (depIx);

   var xMin;
   var xMax;
   var yMin;
   var yMax;

   if (expIx == undefined)
   {
      xMin = wPos;
      xMax = wPos + 1;
      yMin = yMean;
      yMax = yMean;
   }
   else
   {
      var xMean = this.GetMean (expIx);
      var Sxx = this.GetSumSq (expIx, expIx);
      var Sxy = this.GetSumSq (depIx, expIx);
      var wBeta = Sxy / Sxx; // ** the gradient
      xMin = this.Min[expIx];
      xMax = this.Max[expIx];
      yMin = yMean - ( wBeta * (xMean - xMin));
      yMax = yMean + ( wBeta * (xMax - xMean));
   }
   var wString = xMin + ", " + yMin + ", " + xMax + ", " + yMax;
   return wString;

}
// =========================================================================
// Class: cHistogram
// contains all the relevant information for drawing a histogram
// =========================================================================
function cHistogram(iLevel, varIx, iInt, iFixPt) //***  class definition ***
{
   this.Level = iLevel;
   this.FreqString;
   this.CountString;
   this.MaxFreq = -1;
   this.MaxCount = -1;
   this.MaxVal;
   this.CutOff = new Array();
   this.Count = new Array();

   this.Level.SortValues(varIx);
   var wFixPt = iFixPt;
   var wInt = iInt;
   if (wFixPt == undefined)
     wFixPt = this.Level.Min[varIx];
   if (wInt == undefined)
     wInt = (this.Level.Max[varIx] - this.Level.Min[varIx])/10;
   var wzFixPt = wFixPt;
   for (var ix = 1 ; wFixPt > this.Level.sortVals[varIx][0][0]; wFixPt = wzFixPt - ((ix++) * wInt));
   this.CountString = "" + wFixPt + ", 0" ;
   this.FreqString = "" + wFixPt + ", 0" ;
   var valIx = 0;
   wzFixPt = wFixPt;
   for (var ix = 0 ; !(wFixPt > this.Level.Max[varIx]) ; ix++)
   {
      // The following is necessary because JavaScript is bad at arithmetic
      // and introduces rounding errors.
      wFixPt = Math.round( (wzFixPt + (ix * wInt)) * Math.pow(10,6)) / Math.pow(10,6);
      var wCount = 0;
      for (; valIx < this.Level.sortVals[varIx].length &&
         this.Level.sortVals[varIx][valIx][0] < (wFixPt + wInt); valIx++)
         // add count to interval frequency
         wCount += this.Level.sortVals[varIx][valIx][1];
      if (wCount > this.MaxCount)
      {
         this.MaxCount = wCount;
         this.MaxVal = wFixPt;
      }
      this.Count[ix] = wCount;
      this.CutOff[ix] = wFixPt;
      this.CountString +=  ", " + wFixPt + ", " + (wCount);
      this.FreqString +=  ", " + wFixPt + ", " + (wCount/this.Level.ObsCount);
   }
   this.MaxFreq = this.MaxCount/this.Level.ObsCount;
   // ensure vertical bar at RHS:
   this.CountString +=  ", " + wFixPt + ", 0, " + wFixPt + ", 0";
   this.FreqString +=  ", " + wFixPt + ", 0, " + wFixPt + ", 0";

//*** methods ***
   this.Tabulate = cHistogram_Tabulate;

}
// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
function cHistogram_Tabulate()
{
//   document.write("<table style='border-style:solid;border-width:1'>");
   var wTot = 0;
   document.write("<table border=1>");
   document.write("<tr><th>Interval</th><th>Count</th><th>Rel.<br/>Freq.</th></tr>")
   for (var ix=0; ix < this.Count.length - 1; ix++)
   {
      document.write("<tr><td>" + this.CutOff[ix]+ " - " + this.CutOff[ix + 1] + "</td>" +
        "<td style='text-align:right'>" + this.Count[ix] + "</td>" +
        "<td style='text-align:right'>" + this.Count[ix]/this.Level.ObsCount + "</td>" +
        "</tr>")
      wTot += this.Count[ix];
   }
   document.write("<tr><td>Total</td><td style='text-align:right'>" +
      wTot + "</td></tr>")
   document.write("</table>");
}
// =========================================================================
// Library functions
// =========================================================================
// -------------------------------------------------------------------------
// Extension to math class object to calculate modulus of a number by
// a divisor
// -------------------------------------------------------------------------
function Modulus (wNum , wDiv)
{
   var wMod = wNum - (wDiv * Math.floor(wNum/wDiv));
   return wMod;
}
// -------------------------------------------------------------------------
// Function to format number to show n decimal places
// -------------------------------------------------------------------------
function formatSig (iNum,wDP)
{
   if (wDP == undefined)
      wDP = 2;
   var wNeg = false;
   if (iNum < 0)
   {
      iNum *= -1;
      wNeg = true;
   }
   var wNum;
   var sNum;
   var strZeros = "00000000000000000000";
   if (wDP > 20)
      wDP = 20;
   wNum = (1.0 * iNum) + (0.5 * Math.pow(10,-1 * wDP));
   sNum = new String(wNum.toString());
   var ix  = sNum.indexOf("e-");
   if (ix  > -1)
   {
     var wExp = Number(sNum.substr (ix+2));
     tString = new String("0.0000000000000000000000000000000");
     sNum = tString.substr(0,wExp+1) +
        sNum.substr(0,1) + sNum.substr(2,ix-2);
   }
   ix = sNum.indexOf(".");
   sNum = (ix < 0) ?  sNum + "." + strZeros : sNum + strZeros;
   ix = sNum.indexOf(".");
   sNum = (wDP == 0) ? sNum.substr(0,ix + wDP) : sNum.substr(0,ix + wDP + 1);
   if (wNeg == true)
     sNum = "-" + sNum;

   return sNum;
}
// -------------------------------------------------------------------------
// Prints the arguments as a row in a html table
// -------------------------------------------------------------------------
function writeRow ()
{
   document.write ("<tr>");
   for (ix = 0; ix < arguments.length; ix++)
      document.write ("<td style='text-align:right'>" + arguments[ix] + "</td>");
   document.write ("</tr>");

}
// -------------------------------------------------------------------------
// Function to format number to show at least 4 significant figures
// -------------------------------------------------------------------------
function formatSig (iNum,iSig)
{
   wNum = (iNum > 0) ? iNum : (-1 * iNum);
   for (Prec = iSig; Prec > -1 && wNum > 1; Prec--)
      wNum /= 10;
   return formatDec(iNum,Prec);   
}
// -------------------------------------------------------------------------
// Function to format number to show n decimal places
// -------------------------------------------------------------------------
function formatDec (iNum,wDP)
{
   if (wDP == undefined)
      wDP = 2;
   var wNeg = false;
   if (iNum < 0)
   {
      iNum *= -1;
      wNeg = true;
   }
   var wNum;
   var sNum;
   var strZeros = "00000000000000000000";
   if (wDP > 20)
      wDP = 20;
   wNum = (1.0 * iNum) + (0.5 * Math.pow(10,-1 * wDP));
   sNum = new String(wNum.toString());
   var ix  = sNum.indexOf("e-");
   if (ix  > -1)
   {
     var wExp = Number(sNum.substr (ix+2));
     tString = new String("0.0000000000000000000000000000000");
     sNum = tString.substr(0,wExp+1) +
        sNum.substr(0,1) + sNum.substr(2,ix-2);
   }
   ix = sNum.indexOf(".");
   if (ix < 0)
      sNum = sNum + "." + strZeros;
   else
      sNum = sNum + strZeros;
   ix = sNum.indexOf(".");
   if (wDP == 0)
      sNum = sNum.substr(0,ix + wDP);
   else
      sNum = sNum.substr(0,ix + wDP + 1);
   if (wNeg == true)
     sNum = "-" + sNum;

   return sNum;
}

